LOCAL INCLUDE 'INPUT.INC'
C                                       Declarations for inputs
      INTEGER NPARMS
C                                       NPARMS=no. adverbs passed.
C USERID     User number.
C INNAME     Main input file (name).
C INCLASS    Main input file (class).
C INSEQ      Main input file (seq. #).
C INDISK     Disk unit #.       0 => any
C IN2CLASS   Class for images that will be added.
C CUTOFF     Distance cutoff (arcmin) for fields
C INFILE     Field list catalog file, default =
C            /bob1/aips/first/fields/fields.cat
C
      PARAMETER (NPARMS=8)
      INTEGER AVTYPE(NPARMS), AVDIM(2,NPARMS)
      CHARACTER AVNAME(NPARMS)*8
LOCAL END

LOCAL INCLUDE 'INPUTDATA.INC'
C                                       DATA statments defining input
C                                       parameters.
C                                       Adverb names
C                      1        2         3          4        5
      DATA AVNAME /'USERID', 'INNAME', 'INCLASS', 'INSEQ', 'INDISK',
C            6         7        8
     *   'IN2CLASS','CUTOFF', 'INFILE'/
C                                       Adverb data types (PAOOF.INC)
C                      1        2         3          4        5
      DATA AVTYPE /  OOAINT,  OOACAR,   OOACAR,    OOAINT,  OOAINT,
C           6          7        8
     *    OOACAR,  OOARE,    OOACAR/
C                                       Adverb dimensions (as 2D)
C                      1        2         3          4        5
      DATA AVDIM /   1,1,    12,1,      6,1,       1,1,     1,1,
C           6          7        8
     *    6,1,       1,1,    48,1/
LOCAL END

      PROGRAM FLDNN
C-----------------------------------------------------------------------
C! Create NN table giving names and status of nearby FIRST fields
C# Utility OOP
C   This software is the subject of a User agreement and is confidential
C   in nature. It shall not be sold or otherwise made available or
C   disclosed to third parties.
C-----------------------------------------------------------------------
      INTEGER  IERR, BUFF1(256)
      INTEGER NHIST, NHMAX
      PARAMETER (NHMAX = 20)
      CHARACTER PRGM*6, INOBJ*32, OUTOBJ*32, MYHIST(NHMAX)*72
      DATA PRGM /'FLDNN '/, NHIST /0/
C-----------------------------------------------------------------------
C                                       Startup
      CALL FLDNIN (PRGM, INOBJ, OUTOBJ, IERR)
C                                       Do action
      IF (IERR.EQ.0) CALL FLDNAB (INOBJ, OUTOBJ, MYHIST, NHMAX,
     *   NHIST, IERR)
C                                       History
      IF (IERR.EQ.0) CALL FLDNHI (INOBJ, MYHIST, NHIST)
C                                       Close down files, etc.
 990  CALL DIE (IERR, BUFF1)
C
 999  STOP
      END

      SUBROUTINE FLDNIN (PRGN, INOBJ, OUTOBJ, IERR)
C-----------------------------------------------------------------------
C   FLDNIN gets input parameters for FLDNN and creates the input object.
C
C   Inputs:
C      PRGN    C*6  Program name
C   Output:
C      INOBJ   C*32 Input object name
C      OUTOBJ  C*32 Output object name
C      IERR    I    Error code: 0 => ok
C                               4 => user routine detected error.
C                               5 => catalog troubles
C                               8 => can't start
C-----------------------------------------------------------------------
      INTEGER   IERR
      CHARACTER PRGN*6, INOBJ*32, OUTOBJ*32
C
      INTEGER   NKEY1
C                                       NKEY1=no. adverbs to copy to INOBJ
      PARAMETER (NKEY1=8)
      CHARACTER INK1(NKEY1)*8, OUTK1(NKEY1)*32
      INCLUDE 'INCS:DMSG.INC'
      INCLUDE 'INCS:PAOOF.INC'
      INCLUDE 'INPUT.INC'
      INCLUDE 'INPUTDATA.INC'
C                                       Adverbs to copy to INOBJ
C                      1        2         3          4        5
      DATA INK1   /'USERID', 'INNAME', 'INCLASS', 'INSEQ', 'INDISK',
C           6          7         8
     *   'IN2CLASS','CUTOFF', 'INFILE'/
C                                       May rename adverbs to INOBJ
C                      1        2         3          4        5
      DATA OUTK1  /'USERID', 'NAME',   'CLASS',   'IMSEQ',   'DISK',
C           6         7         8
     *   'IN2CLASS','CUTOFF', 'INFILE'/
C
C-----------------------------------------------------------------------
C                                       Startup,  returns "Input" object
C                                       containing POPS adverbs
      CALL AV2INP (PRGN, NPARMS, AVNAME, AVTYPE, AVDIM, 'Input', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Create input object
      INOBJ = 'Input object'
      CALL CREATE (INOBJ, 'IMAGE', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Copy adverbs to object
      CALL IN2OBJ ('Input', NKEY1, INK1, OUTK1, INOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Output object is the table
      OUTOBJ = 'Output table'
      CALL CREATE (OUTOBJ, 'TABLE', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Copy adverbs to object
      CALL IN2OBJ ('Input', NKEY1, INK1, OUTK1, OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C
 999  RETURN
      END

      SUBROUTINE FLDNHI (OUTOBJ, MYHIST, NHIST)
C-----------------------------------------------------------------------
C   Routine to write history file to output object.  This assumes
C   that a previous history exists and merely adds the information from
C   the current task.
C   Inputs:
C      OUTOBJ  C*?  Output object
C      MYHIST  C*72 Array (NHIST) of extra history lines
C-----------------------------------------------------------------------
      INTEGER NHIST
      CHARACTER OUTOBJ*(*), MYHIST(NHIST)*72
C
      INTEGER   NADV, I
      PARAMETER (NADV=6)
      CHARACTER LIST(NADV)*8
      INTEGER   IERR
      INCLUDE 'INCS:DMSG.INC'
C                                       Adverbs to copy to history
      DATA LIST /'INNAME', 'INCLASS', 'INSEQ', 'IN2CLASS',
     *   'CUTOFF', 'INFILE'/
C-----------------------------------------------------------------------
C                                       Add task label to history
      CALL OHTIME (OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 990
C                                       Copy adverb values.
      CALL OHLIST ('Input', LIST, NADV, OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 990
C                                       Add info on results.
      DO 10 I=1,NHIST
         CALL OHWRIT(MYHIST(I), OUTOBJ, IERR)
         IF (IERR.NE.0) GO TO 990
  10  CONTINUE
C
      CALL OCLOSE (OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 990
      GO TO 999
C                                       Error
 990  MSGTXT = 'ERROR WRITING HISTORY FOR ' // OUTOBJ
      CALL MSGWRT (4)
 999  RETURN
      END

      SUBROUTINE FLDNAB (INOBJ, OUTOBJ, MYHIST, NHMAX, NHIST, IERR)
C-----------------------------------------------------------------------
C   Read database and create NN table
C   Inputs:
C      INOBJ   C*   Name of input object.
C      OUTOBJ  C*   Name of output object.
C      NHMAX   I    Maximum number of history lines to write.
C   Output:
C      MYHIST  C*72 History lines.
C      NHIST   I    Number of history lines written.
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      INTEGER   NHMAX, NHIST, IERR
      CHARACTER INOBJ*(*), OUTOBJ*(*), MYHIST(NHMAX)*72
C
C                                       MAXFLD = max. no. fields
      INTEGER   MAXFLD
      PARAMETER (MAXFLD = 200)
C
      DOUBLE PRECISION RA(MAXFLD), DEC(MAXFLD)
      CHARACTER*12     NAME(MAXFLD)
      CHARACTER*1      STATUS(MAXFLD)
C                                       FLFILE = name of field file
      CHARACTER*48 FLFILE
C
      INTEGER   I, TYPE, DIM(7), NFLD, LOWCAS, NAXIS(7)
      REAL CUTOFF, XSHIFT, YSHIFT, DUMMY, CDELT(7),
     *   XYZI(7), XYZO(7)
      REAL GCIRCL, SEP
      DOUBLE PRECISION RACEN, DECEN, RA2000, DC2000
      CHARACTER REFNAM*12, CDUMMY*1, IN2CL*6
C
      INTEGER          UNIT, IRECNO, EQUINX
C
      INCLUDE 'INCS:PAOOF.INC'
      INCLUDE 'INCS:DMSG.INC'
      DATA XYZI/7*0.0/
C-----------------------------------------------------------------------
C                                       Error if no IN2CLASS given
      CALL OGET (INOBJ, 'IN2CLASS', TYPE, DIM, DUMMY, IN2CL, IERR)
      IF (IERR.NE.0) GO TO 999
      IF (IN2CL.EQ.'      ') THEN
         MSGTXT = 'ERROR: IN2CLASS MAY NOT BE BLANK'
         CALL MSGWRT (4)
         IERR = 1
         GO TO 999
      ENDIF
C                                       Open input object
      CALL OOPEN (INOBJ, 'READ', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Get input parameters
      CALL OGET (INOBJ, 'NAME', TYPE, DIM, DUMMY, REFNAM, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET (INOBJ, 'CUTOFF', TYPE, DIM, CUTOFF, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Put CUTOFF in degrees
      CUTOFF = CUTOFF / 60.0
C
      CALL OGET (INOBJ, 'INFILE', TYPE, DIM, DUMMY, FLFILE, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Convert catalog name to lower case
      LOWCAS = ICHAR('a') - ICHAR('A')
      DO 50 I=1,LEN(FLFILE)
         IF (FLFILE(I:I).GE.'A' .AND. FLFILE(I:I).LE.'Z') THEN
            FLFILE(I:I) = CHAR( ICHAR(FLFILE(I:I)) + LOWCAS )
         END IF
50    CONTINUE
C                                       Get position & equinox
      CALL GETPOS (INOBJ, RACEN, DECEN, XSHIFT, YSHIFT, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL GETEQN (INOBJ, EQUINX, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Convert RACEN, DECEN to J2000
      RA2000 = RACEN
      DC2000 = DECEN
      IF (EQUINX.NE.2000) THEN
         CALL PRECSS(RA2000,DC2000,EQUINX,2000,IERR)
         IF (IERR.NE.0) GO TO 999
      ENDIF
C                                       Call pixel position routine
C                                       to force initialization
      CALL PSNCVT(INOBJ, XYZI, INOBJ, XYZO, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Get image dimensions & scale
      CALL OGET (INOBJ, 'ARRAY.ARRAY_DESC.NAXIS', TYPE, DIM,
     *   NAXIS, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET (INOBJ, 'IMAGE_DESC.CDELT', TYPE, DIM,
     *   CDELT, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Clear file status
      CALL IMCCLR (INOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Close input object
      CALL OCLOSE (INOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Open field catalog
      UNIT = 11
      CALL FLDOPN (UNIT, FLFILE, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Find first field
      CALL FLDFND (UNIT, RA2000, DC2000, NAXIS(1), NAXIS(2), 
     *   ABS(CDELT(1)), ABS(CDELT(2)), CUTOFF, IRECNO,
     *   NAME(1), RA(1), DEC(1), IERR)
      IF (IERR.NE.0) GO TO 999
C                                       WHILE Loop to find all fields
      NFLD = 0
  60  CONTINUE
         IF (IRECNO.LE.0) GO TO 65
C                                       Don't include reference field
         IF (NAME(NFLD+1).NE.REFNAM) NFLD = NFLD+1
         IF (NFLD.EQ.MAXFLD) THEN
            MSGTXT = 'WARNING: Too many fields found, cutting off list'
            CALL MSGWRT (4)
            GO TO 65
         ENDIF
         CALL FLDNXT (UNIT, RA2000, DC2000, NAXIS(1), NAXIS(2), 
     *      ABS(CDELT(1)), ABS(CDELT(2)), CUTOFF, IRECNO,
     *      NAME(NFLD+1), RA(NFLD+1), DEC(NFLD+1), IERR)
         IF (IERR.NE.0) GO TO 999
         GO TO 60
C
  65  CONTINUE
      WRITE (MSGTXT,1003) 'Found ',NFLD,' fields closer than',CUTOFF
      CALL MSGWRT (1)
C                                       Close field catalog
      CALL FLDCLS (UNIT, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Open output table
      CALL NNOPEN (OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Set status for fields
      CALL STATST (OUTOBJ, NFLD, NAME, STATUS, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Write fields to table
      DIM(2) = 1
      DIM(3) = 0
      DO 100 I = 1,NFLD
C                                       Name
         DIM(1) = 12
         CALL TABDPT (OUTOBJ, I, 1, OOACAR, DIM, DUMMY, NAME(I), IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Status
         DIM(1) = 1
         CALL TABDPT (OUTOBJ, I, 2, OOACAR, DIM, DUMMY, STATUS(I), IERR) 
         IF (IERR.NE.0) GO TO 999
C                                       RA
         DIM(1) = 1
         CALL TABDPT (OUTOBJ, I, 3, OOADP, DIM, RA(I), CDUMMY, IERR) 
         IF (IERR.NE.0) GO TO 999
C                                       Dec
         DIM(1) = 1
         CALL TABDPT (OUTOBJ, I, 4, OOADP, DIM, DEC(I), CDUMMY, IERR) 
         IF (IERR.NE.0) GO TO 999
C                                       Separation between field centers
         SEP = GCIRCL(RA(I),DEC(I),RA2000,DC2000)
         DIM(1) = 1
         CALL TABDPT (OUTOBJ, I, 5, OOARE, DIM, SEP, CDUMMY, IERR) 
         IF (IERR.NE.0) GO TO 999
 100  CONTINUE
C
      IF (NHIST.LT.NHMAX) THEN
         NHIST = NHIST+1
         WRITE(MYHIST(NHIST),1004)
     *      'Created NN extension with', NFLD, ' entries'
      ENDIF
C                                       Close output table
      CALL OCLOSE (OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Sort table by separation
      CALL TBLSRT (OUTOBJ, 'DISTANCE', 'RA', IERR)
      IF (IERR.NE.0) GO TO 999
C
 999  RETURN
C-----------------------------------------------------------------------
1001  FORMAT (A5,I1)
1002  FORMAT (A5,I2)
1003  FORMAT (A,I4,A,F10.4)
1004  FORMAT (A,I4,A)
1005  FORMAT (A,I4,A)
      END

      SUBROUTINE STATST (OUTOBJ, NFLD, NAME, STATUS, IERR)
C-----------------------------------------------------------------------
C   STATST checks the passed list of fields to see whether they are
C   currently on disk or not and sets the STATUS flag accordingly.  It
C   also checks to see whether a NN table already exists; if a field
C   already has an entry and the STATUS flag is set to 'Y' (meaning it
C   has already been added into the map) then the 'Y' status is retained.
C   It is an error if there are fields listed in the existing NN table
C   that are not in the NAME list.
C
C   Inputs:
C      OUTOBJ  C    Name of output table object
C      NFLD    I    Number of fields
C      NAME    C    Names of fields
C   Outputs:
C      STATUS  C    Status flags for each field.  Set to:
C                      'E' if image for this field exists on disk
C                      'N' if image for this field is not on disk
C                      'Y' if STATUS for this field is in existing
C                          NN table and is set to 'Y'
C      IERR    I    Error code: 0 => ok
C                           other => error
C-----------------------------------------------------------------------
      INTEGER       NFLD, IERR
      CHARACTER*(*) OUTOBJ, NAME(NFLD), STATUS(NFLD)
C
      INTEGER   TYPE, DIM(7)
      INTEGER I, J, OLDNF
      LOGICAL EXTOBJ
      REAL DUMMY
      CHARACTER CDUMMY, ONAME*12, OSTAT*1, IN2CL*6, TSTOBJ*32
      INCLUDE 'INCS:DMSG.INC'
      INCLUDE 'INCS:PAOOF.INC'
C                                       Initialize all status flags
      DO 10 I=1,NFLD
         STATUS(I) = 'N'
   10 CONTINUE
C                                       Check whether NN extension already
C                                       has data by getting number of rows
C                                       already in table.
      CALL OGET (OUTOBJ, 'NROW', TYPE, DIM, OLDNF, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Set status of old fields
      DO 20 I=1,OLDNF
         CALL TABDGT (OUTOBJ, I, 1, OOACAR, DIM, DUMMY, ONAME, IERR)
         IF (IERR.NE.0) GO TO 999
         CALL TABDGT (OUTOBJ, I, 2, OOACAR, DIM, DUMMY, OSTAT, IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Find this field in list
         DO 30 J=1,NFLD
            IF (NAME(J).EQ.ONAME) THEN
               IF (OSTAT.EQ.'Y') STATUS(J) = 'Y'
               GO TO 20
            ENDIF
   30    CONTINUE
C                                       Error if field not found
         WRITE (MSGTXT, 1001)
     *     'Old field ' // ONAME // ' is not in current field list'
         CALL MSGWRT (4)
         IERR = 1
         GO TO 999
C
   20 CONTINUE
C                                       All 'Y' status flags have been restored
C                                       Check all other fields for existence
C
C                                       Set up image object for testing
      TSTOBJ = 'Existence test object'
      CALL CREATE (TSTOBJ, 'IMAGE', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Get class for image files
      CALL OGET (OUTOBJ, 'IN2CLASS', TYPE, DIM, DUMMY, IN2CL, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OPUT (TSTOBJ, 'CLASS', TYPE, DIM, DUMMY, IN2CL, IERR)
      IF (IERR.NE.0) GO TO 999
C
      DO 40 I=1,NFLD
         IF (STATUS(I).NE.'Y') THEN
C                                       Use default SEQ, DISK
            DIM(1) = 1
            DIM(2) = 1
            DIM(3) = 0
            CALL OPUT (TSTOBJ, 'IMSEQ', OOAINT, DIM, 0, CDUMMY, IERR)
            IF (IERR.NE.0) GO TO 999
            CALL OPUT (TSTOBJ, 'DISK', OOAINT, DIM, 0, CDUMMY, IERR)
            IF (IERR.NE.0) GO TO 999
C                                       Set name
            DIM(1) = LEN(NAME(I))
            DIM(2) = 1
            DIM(3) = 0
            CALL OPUT (TSTOBJ, 'NAME', OOACAR, DIM, DUMMY,
     *         NAME(I), IERR)
            IF (IERR.NE.0) GO TO 999
C                                       Set existence flag if file exists
            IF (EXTOBJ(TSTOBJ)) STATUS(I) = 'E'
         ENDIF
   40 CONTINUE
C                                       Destroy the test object
      CALL DESTRY(TSTOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
 999  RETURN
C-----------------------------------------------------------------------
1001  FORMAT (A)
1002  FORMAT (A5,I1)
1003  FORMAT (A5,I2)
      END

      SUBROUTINE NNOPEN (OUTOBJ, IERR)
C-----------------------------------------------------------------------
C   Create and open NN table.
C   Inputs:
C      OUTOBJ   C*   Name of output table object.
C   Output:
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      CHARACTER OUTOBJ*(*)
      INTEGER   IERR
C                                       TLABEL = table label
      CHARACTER*56 TLABEL
      PARAMETER (TLABEL =
     *   'NN table: nearest-neighbor fields in FIRST survey.')
C                                       NCOL = number of columns
      INTEGER NCOL
      PARAMETER (NCOL = 5)
C                                       CLABEL = column labels
      CHARACTER CLABEL*(24*NCOL)
C                                       CUNIT = column units
      CHARACTER CUNIT*(8*NCOL)
C                                       CTYPE = column types
      INTEGER CTYPE(NCOL)
C                                       CDIM = column dimensions
      INTEGER CDIM(NCOL)
C                                       TTYPE = table type
      CHARACTER*2 TTYPE
C
      INTEGER   DIM(7)
      REAL      DUMMY
      CHARACTER CDUMMY
C
      INCLUDE 'INCS:PAOOF.INC'
C-----------------------------------------------------------------------
C                                       Table label
      DIM(1) = LEN(TLABEL)
      DIM(2) = 1
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'LABEL', OOACAR, DIM, DUMMY, TLABEL, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Number of columns
      DIM(1) = 1
      DIM(2) = 1
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'NCOL', OOAINT, DIM, NCOL, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Column labels
      CLABEL(24*1-23:24*1) = 'FLDNAME'
      CLABEL(24*2-23:24*2) = 'STATUS'
      CLABEL(24*3-23:24*3) = 'RA'
      CLABEL(24*4-23:24*4) = 'DEC'
      CLABEL(24*5-23:24*5) = 'DISTANCE'
      DIM(1) = 24
      DIM(2) = NCOL
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'COLABEL', OOACAR, DIM, DUMMY, CLABEL, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Column units
      CUNIT(8*1-7:8*1) = ' '
      CUNIT(8*2-7:8*2) = ' '
      CUNIT(8*3-7:8*3) = 'DEGREES'
      CUNIT(8*4-7:8*4) = 'DEGREES'
      CUNIT(8*5-7:8*5) = 'DEGREES'
      DIM(1) = 8
      DIM(2) = NCOL
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'COLUNIT', OOACAR, DIM, DUMMY, CUNIT, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Column types
      CTYPE(1) = OOACAR
      CTYPE(2) = OOACAR
      CTYPE(3) = OOADP
      CTYPE(4) = OOADP
      CTYPE(5) = OOARE
      DIM(1) = NCOL
      DIM(2) = 1
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'COLTYPE', OOAINT, DIM, CTYPE, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Column dimensions
      CDIM(1) = 12
      CDIM(2) = 1
      CDIM(3) = 1
      CDIM(4) = 1
      CDIM(5) = 1
      DIM(1) = NCOL
      DIM(2) = 1
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'COLDIM', OOAINT, DIM, CDIM, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Set version number to 1
      DIM(1) = 1
      DIM(2) = 1
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'VER', OOAINT, DIM, 1, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Set table type to 'NN'
      TTYPE = 'NN'
      DIM(1) = 2
      DIM(2) = 1
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'TBLTYPE', OOACAR, DIM, DUMMY, TTYPE, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Create output table
      CALL TABOPN (OUTOBJ, 'WRIT', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Set sort order to none
      CDIM(1) = 0
      CDIM(2) = 0
      DIM(1) = 2
      DIM(2) = 1
      DIM(3) = 0
      CALL OPUT (OUTOBJ, 'SORT', OOAINT, DIM, CDIM, CDUMMY, IERR)
      IF (IERR.NE.0) GO TO 999
C
 999  RETURN
      END
