LOCAL INCLUDE 'INPUT.INC'
C                                       Declarations for inputs
      INTEGER NPARMS
C                                       NPARMS=no. adverbs passed.
C USERID     User number.
C INNAME     Main input file (name).
C INCLASS    Main input file (class).
C INSEQ      Main input file (seq. #).
C INDISK     Disk unit #.       0 => any
C INVERS     Input  file version no.
C CELLSIZE   (X,Y) size of grid in asec
C FLDSIZE    (X,Y) size of central field in pixels
C CUTOFF     Flux cutoff for sources
C INFILE     SRCFN database, default = /bob1/aips/first/srcfn/
C
      PARAMETER (NPARMS=10)
      INTEGER AVTYPE(NPARMS), AVDIM(2,NPARMS)
      CHARACTER AVNAME(NPARMS)*8
LOCAL END

LOCAL INCLUDE 'INPUTDATA.INC'
      INCLUDE 'INCS:PUVD.INC'
C                                       DATA statments defining input
C                                       parameters.
C                                       Adverb names
C                      1        2         3          4        5
      DATA AVNAME /'USERID', 'INNAME', 'INCLASS', 'INSEQ', 'INDISK',
C           6         7        8           9         10
     *   'INVER','CELLSIZE', 'FLDSIZE', 'CUTOFF','INFILE'/
C                                       Adverb data types (PAOOF.INC)
C                      1        2         3          4        5
      DATA AVTYPE /  OOAINT,  OOACAR,   OOACAR,    OOAINT,  OOAINT,
C           6          7        8         9         10
     *    OOAINT,     OOARE,  OOAINT,    OOARE,    OOACAR/
C                                       Adverb dimensions (as 2D)
C                      1        2         3          4        5
      DATA AVDIM /   1,1,    12,1,      6,1,       1,1,     1,1,
C           6          7        8         9         10
     *    1,1,       2,1,    2,MAXAFL,  1,1,      48,1/
LOCAL END

      PROGRAM SCTBX
C-----------------------------------------------------------------------
C! Set keywords giving MX field locations from SRCFN catalog
C# Utility OOP
C   This software is the subject of a User agreement and is confidential
C   in nature. It shall not be sold or otherwise made available or
C   disclosed to third parties.
C-----------------------------------------------------------------------
      INTEGER  IERR, BUFF1(256)
      INTEGER NHIST, NHMAX
      PARAMETER (NHMAX = 20)
      CHARACTER PRGM*6, INOBJ*32, MYHIST(NHMAX)*72
      DATA PRGM /'SCTBX '/, NHIST /0/
C-----------------------------------------------------------------------
C                                       Startup
      CALL CTBXIN (PRGM, INOBJ, IERR)
C                                       Do action
      IF (IERR.EQ.0) CALL CTBXAB (INOBJ, MYHIST, NHMAX,
     *   NHIST, IERR)
C                                       History
      IF (IERR.EQ.0) CALL CTBXHI (INOBJ, MYHIST, NHIST)
C                                       Close down files, etc.
 990  CALL DIE (IERR, BUFF1)
C
 999  STOP
      END

      SUBROUTINE CTBXIN (PRGN, INOBJ, IERR)
C-----------------------------------------------------------------------
C   CTBXIN gets input parameters for SCTBX and creates the input object.
C
C   Inputs:
C      PRGN    C*6  Program name
C   Output:
C      INOBJ   C*32 Input object name
C      IERR    I    Error code: 0 => ok
C                               4 => user routine detected error.
C                               5 => catalog troubles
C                               8 => can't start
C-----------------------------------------------------------------------
      INTEGER   IERR
      CHARACTER PRGN*6, INOBJ*32
C
      INTEGER   NKEY1
C                                       NKEY1=no. adverbs to copy to INOBJ
      PARAMETER (NKEY1=10)
      CHARACTER INK1(NKEY1)*8, OUTK1(NKEY1)*32
      INCLUDE 'INCS:DMSG.INC'
      INCLUDE 'INCS:PAOOF.INC'
      INCLUDE 'INPUT.INC'
      INCLUDE 'INPUTDATA.INC'
C                                       Adverbs to copy to INOBJ
C                      1        2         3          4        5
      DATA INK1   /'USERID', 'INNAME', 'INCLASS', 'INSEQ', 'INDISK',
C           6         7        8          9         10
     *   'INVER', 'CELLSIZE','FLDSIZE','CUTOFF', 'INFILE'/
C                                       May rename adverbs to INOBJ
C                      1        2         3          4        5
      DATA OUTK1  /'USERID', 'NAME',   'CLASS',   'IMSEQ',   'DISK',
C           6         7        8          9         10
     *   'VER',   'CELLSIZE', 'FLDSIZE','CUTOFF', 'INFILE'/
C
C-----------------------------------------------------------------------
C                                       Startup,  returns "Input" object
C                                       containing POPS adverbs
      CALL AV2INP (PRGN, NPARMS, AVNAME, AVTYPE, AVDIM, 'Input', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Create input object
      INOBJ = 'Input object'
      CALL CREATE (INOBJ, 'IMAGE', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Copy adverbs to object
      CALL IN2OBJ ('Input', NKEY1, INK1, OUTK1, INOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C
 999  RETURN
      END

      SUBROUTINE CTBXHI (OUTOBJ, MYHIST, NHIST)
C-----------------------------------------------------------------------
C   Routine to write history file to output object.  This assumes
C   that a previous history exists and merely adds the information from
C   the current task.
C   Inputs:
C      OUTOBJ  C*?  Output object
C      MYHIST  C*72 Array (NHIST) of extra history lines
C-----------------------------------------------------------------------
      INTEGER NHIST
      CHARACTER OUTOBJ*(*), MYHIST(NHIST)*72
C
      INTEGER   NADV, I
      PARAMETER (NADV=8)
      CHARACTER LIST(NADV)*8
      INTEGER   IERR
      INCLUDE 'INCS:DMSG.INC'
C                                       Adverbs to copy to history
      DATA LIST /'INNAME', 'INCLASS', 'INSEQ', 'INVER',
     *   'CELLSIZE', 'FLDSIZE', 'CUTOFF', 'INFILE'/
C-----------------------------------------------------------------------
C                                       Add task label to history
      CALL OHTIME (OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 990
C                                       Copy adverb values.
      CALL OHLIST ('Input', LIST, NADV, OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 990
C
C                                       Add info on results.
      DO 10 I=1,NHIST
         CALL OHWRIT(MYHIST(I), OUTOBJ, IERR)
         IF (IERR.NE.0) GO TO 990
  10  CONTINUE
C
      CALL OCLOSE (OUTOBJ, IERR)
      IF (IERR.NE.0) GO TO 990
      GO TO 999
C                                       Error
 990  MSGTXT = 'ERROR WRITING HISTORY FOR ' // OUTOBJ
      CALL MSGWRT (4)
 999  RETURN
      END

      SUBROUTINE CTBXAB (INOBJ, MYHIST, NHMAX, NHIST, IERR)
C-----------------------------------------------------------------------
C   Read database and write box keywords to header
C   Inputs:
C      INOBJ   C*   Name of input object.
C      NHMAX   I    Maximum number of history lines to write.
C   Output:
C      MYHIST  C*72 History lines.
C      NHIST   I    Number of history lines written.
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      INTEGER   NHMAX, NHIST, IERR
      CHARACTER INOBJ*(*), MYHIST(NHMAX)*72
C
C
C                                       MAXSRC = max. no. sources
      INTEGER   MAXSRC
      PARAMETER (MAXSRC = 1000)
C                                       MAXBOX = max. no. boxes
      INTEGER   MAXBOX
      PARAMETER (MAXBOX = 16)
C                                       SRCDIR = name of database directory
      CHARACTER*48 SRCDIR
C                                       DEFDIR = default name of directory
      CHARACTER*48 DEFDIR
      PARAMETER (DEFDIR = '/bob1/aips/first/srcfn')
C                                       SRCNAM = name used for source file
      CHARACTER*12 SRCNAM
      CHARACTER*8 KEYWDX, KEYWDY, KEYWDL, KEYWDF
C
      INTEGER   TYPE, DIM(7), NSRC, NBOX, LOWCAS, FLDSIZ(2,64)
      REAL CELLSZ(2), CUTOFF
      REAL XSRC(MAXSRC), YSRC(MAXSRC), XSIZ(MAXSRC), YSIZ(MAXSRC),
     *   FSRC(MAXSRC), XBOX(MAXBOX), YBOX(MAXBOX),
     *   FBOX(MAXBOX), XSHIFT, YSHIFT
      DOUBLE PRECISION RA(MAXSRC), DEC(MAXSRC), RACEN, DECEN,
     *   DWORK(MAXSRC)
      INTEGER I, LBOX(MAXBOX), IWORK(MAXSRC)
      LOGICAL ALLBLK
C
      INCLUDE 'INCS:PAOOF.INC'
      INCLUDE 'INCS:DMSG.INC'
C-----------------------------------------------------------------------
C                                       Open input object
      CALL OOPEN (INOBJ, 'READ', IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Input parameters
      CALL OGET (INOBJ, 'CUTOFF', TYPE, DIM, CUTOFF, CUTOFF, IERR)
      IF (IERR.NE.0) GO TO 999
C
      CALL OGET (INOBJ, 'CELLSIZE', TYPE, DIM, CELLSZ, CELLSZ, IERR)
      IF (IERR.NE.0) GO TO 999
      CELLSZ(1) = ABS(CELLSZ(1))
      CELLSZ(2) = ABS(CELLSZ(2))
      IF (CELLSZ(2) .EQ. 0.0) CELLSZ(2) = CELLSZ(1)
C
      CALL OGET (INOBJ, 'FLDSIZE', TYPE, DIM, FLDSIZ, FLDSIZ, IERR)
      IF (IERR.NE.0) GO TO 999
C
      CALL OGET (INOBJ, 'INFILE',TYPE, DIM, SRCDIR, SRCDIR, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET (INOBJ, 'NAME',TYPE, DIM, SRCNAM, SRCNAM, IERR)
      IF (IERR.NE.0) GO TO 999
C
C                                       Convert database directory to lowercase
      ALLBLK = .TRUE.
      LOWCAS = ICHAR('a') - ICHAR('A')
      DO 50 I=1,LEN(SRCDIR)
         IF (SRCDIR(I:I).GE.'A' .AND. SRCDIR(I:I).LE.'Z') THEN
            SRCDIR(I:I) = CHAR( ICHAR(SRCDIR(I:I)) + LOWCAS )
         END IF
         IF (SRCDIR(I:I).NE.' ') ALLBLK = .FALSE.
50    CONTINUE
C                                       If SRCDIR blank, use default
      IF (ALLBLK) SRCDIR = DEFDIR
C                                       Convert SRCNAM to lowercase too
      DO 60 I=1,LEN(SRCNAM)
         IF (SRCNAM(I:I).GE.'A' .AND. SRCNAM(I:I).LE.'Z') THEN
            SRCNAM(I:I) = CHAR( ICHAR(SRCNAM(I:I)) + LOWCAS )
         END IF
60    CONTINUE
C
C                                       Read sources from file (if it exists)
      CALL SRCRD (SRCDIR, SRCNAM, MAXSRC,
     *   NSRC, RA, DEC, XSIZ, YSIZ, FSRC, IERR)
      IF (IERR.NE.0) GO TO 999
C
C                                       Check for sources fainter than CUTOFF
      DO 70 I=1,NSRC
         IF (FSRC(I) .LT. CUTOFF) THEN
            WRITE(MSGTXT,1003) NSRC-I+1,CUTOFF
            CALL MSGWRT (1)
            NSRC = I-1
            GO TO 80
         ENDIF
  70  CONTINUE
  80  CONTINUE
C                                       NSRC < 0 means no source file
C                                       NSRC = 0 means no bright sources
      IF (NSRC .LE. 0) THEN
         CALL PUTKEY (INOBJ, 'NCATFLDS', OOAINT, NSRC, NSRC, IERR)
         IF (IERR.NE.0) GO TO 999
         IF (NHIST.LT.NHMAX) THEN
            NHIST = NHIST+1
            WRITE(MYHIST(NHIST),1004) 'Wrote NCATFLDS = ',NSRC
         ENDIF
         GO TO 999
      ENDIF
C                                       Get phase center RA, Dec
      CALL GETPOS(INOBJ, RACEN, DECEN, XSHIFT, YSHIFT, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Sort by flux
      CALL SORT5 (NSRC, FSRC, XSIZ, YSIZ, RA, DEC, DWORK, IWORK, IERR)
C
C                                       Convert RA, Dec to X,Y from center
      DO 90 I=1,NSRC
         CALL SKY2XY(RA(I),DEC(I),RACEN,DECEN,XSRC(I),YSRC(I),IERR)
         IF (IERR.NE.0) GO TO 999
  90  CONTINUE
C                                       Make list of boxes
      CALL BOXLST (NSRC, XSRC, YSRC, XSIZ, YSIZ, FSRC, CELLSZ,
     *   FLDSIZ(1,1), FLDSIZ(2,1), MAXBOX, NBOX, XBOX, YBOX,
     *   LBOX, FBOX, IWORK, DWORK, IERR)
      IF (IERR.NE.0) GO TO 999
C
      WRITE (MSGTXT,1005) 'Required ',NBOX,' boxes'
      CALL MSGWRT (1)
C                                       Write boxes to header
      CALL PUTKEY (INOBJ, 'NCATFLDS', OOAINT, NBOX, NBOX, IERR)
      IF (IERR.NE.0) GO TO 999
      DO 100 I=1,NBOX
         IF (I .LT. 10) THEN
            WRITE(KEYWDX,1001) 'RA', I
            WRITE(KEYWDY,1001) 'DC', I
            WRITE(KEYWDL,1001) 'SZ', I
            WRITE(KEYWDF,1001) 'FL', I
         ELSE
            WRITE(KEYWDX,1002) 'RA', I
            WRITE(KEYWDY,1002) 'DC', I
            WRITE(KEYWDL,1002) 'SZ', I
            WRITE(KEYWDF,1002) 'FL', I
         ENDIF
         CALL PUTKEY (INOBJ, KEYWDX, OOARE,  XBOX(I), XBOX(I), IERR)
         IF (IERR.NE.0) GO TO 999
         CALL PUTKEY (INOBJ, KEYWDY, OOARE,  YBOX(I), YBOX(I), IERR)
         IF (IERR.NE.0) GO TO 999
         CALL PUTKEY (INOBJ, KEYWDL, OOAINT, LBOX(I), LBOX(I), IERR)
         IF (IERR.NE.0) GO TO 999
         CALL PUTKEY (INOBJ, KEYWDF, OOARE,  FBOX(I), FBOX(I), IERR)
         IF (IERR.NE.0) GO TO 999
 100  CONTINUE
      IF (NHIST.LT.NHMAX) THEN
         NHIST = NHIST+1
         WRITE(MYHIST(NHIST),1004)
     *      'Wrote CATRA,DC,SZ,FL for NCATFLDS = ',NBOX
      ENDIF
C                                       Clear file status
      CALL IMCCLR (INOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Close input table
      CALL OCLOSE (INOBJ, IERR)
      IF (IERR.NE.0) GO TO 999
C
 999  RETURN
C-----------------------------------------------------------------------
1001  FORMAT ('CAT',A2,I1)
1002  FORMAT ('CAT',A2,I2)
1003  FORMAT ('Ignoring ',I8,' sources fainter than ',F10.5,' Jy')
1004  FORMAT (A,I4)
1005  FORMAT (A,I4,A)
      END

      SUBROUTINE SRCRD (SRCDIR, SRCNAM, MAXSRC,
     *   NSRC, RA, DEC, XSIZ, YSIZ, FSRC, IERR)
      CHARACTER*(*) SRCDIR, SRCNAM
      INTEGER MAXSRC, NSRC, IERR
      DOUBLE PRECISION RA(MAXSRC), DEC(MAXSRC)
      REAL XSIZ(MAXSRC), YSIZ(MAXSRC), FSRC(MAXSRC)
C
      CHARACTER*61 SRCFIL
      INTEGER I, J, LSRC
      INTEGER RAHH, RAMM, DCDD, DCMM
      REAL RASS, DCSS
      CHARACTER*1 DCSIGN
      LOGICAL FEXIST
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C                                       Build the filename
      SRCFIL = SRCDIR
      DO 100 I=LEN(SRCDIR),1,-1
         IF (SRCDIR(I:I).NE.' ') THEN
            J = I+1
            GO TO 200
         ENDIF
 100  CONTINUE
 200  CONTINUE
C                                       Add filename after directory
      SRCFIL(J:) = '/' // SRCNAM
C                                       Find length of SRCFIL
      DO 300 I=LEN(SRCFIL),1,-1
         IF (SRCFIL(I:I).NE.' ') THEN
            LSRC = I
            GO TO 400
         ENDIF
 300  CONTINUE
 400  CONTINUE
C                                       Replace embedded blanks
      DO 500 I=J,LSRC
         IF (SRCFIL(I:I).EQ.' ') SRCFIL(I:I) = '_'
 500  CONTINUE
C                                       Does file exist?
      INQUIRE (FILE=SRCFIL, EXIST=FEXIST)
      IF (.NOT. FEXIST) THEN
         NSRC = -1
         MSGTXT = 'WARNING: Source file ' // SRCFIL(1:LSRC) //
     *      ' does not exist.'
         CALL MSGWRT (1)
         GO TO 999
      ENDIF
C                                       Read the file
      OPEN (UNIT=55, FILE=SRCFIL, STATUS='OLD', ERR=799)
      NSRC = 0
      DO 600 I = 1,MAXSRC
C                                       Read position from the file
         READ(55,1001,ERR=899,END=700) RAHH, RAMM, RASS,
     *      DCSIGN, DCDD, DCMM, DCSS, XSIZ(I), YSIZ(I), FSRC(I)
         NSRC = I
C                                       Convert HMS to degrees
         CALL HMS2RA(RAHH,RAMM,RASS,RA(I),IERR)
         IF (IERR .NE. 0) GO TO 999
C                                       Convert DMS to degrees
         CALL DMS2DC(DCSIGN,DCDD,DCMM,DCSS,DEC(I),IERR)
         IF (IERR .NE. 0) GO TO 999
 600  CONTINUE
C                                       Too many sources
      MSGTXT = 'WARNING: Reached maximum number of sources'
      CALL MSGWRT (1)
C                                       Normal end-of-file
 700  CONTINUE
      CLOSE (UNIT=55)
      WRITE (MSGTXT,1000) NSRC, SRCFIL(1:LSRC)
      CALL MSGWRT (1)
      IERR = 0
      GO TO 999
C
 799  CONTINUE
      WRITE (MSGTXT,1002) SRCFIL(1:LSRC)
      CALL MSGWRT (8)
      IERR = 5
      GO TO 999
C
 899  CONTINUE
      WRITE (MSGTXT,1003) SRCFIL(1:LSRC)
      CALL MSGWRT (8)
      IERR = 5
      GO TO 999
C
 999  RETURN
C
 1000 FORMAT('Read',I5,' sources from file ',A)
 1001 FORMAT (1X, I2.2, I3.2, F7.3, 1X, A1, I2.2, I3.2, F6.2, 2X,
     *   2F10.3, F13.5)
 1002 FORMAT ('ERROR OPENING SOURCE FILE ',A)
 1003 FORMAT ('ERROR WHILE READING FROM SOURCE FILE ',A)
      END

      SUBROUTINE BOXLST (NSRC, XSRC, YSRC, XSIZ, YSIZ, FSRC, CELLSZ,
     *   NX, NY, MAXBOX, NBOX, XBOX, YBOX, LBOX, FBOX,
     *   IWORK, WORK, IERR)
C-----------------------------------------------------------------------
C   BOXLST makes a list of boxes from a list of sources and their sizes.
C   This is currently a dummy version that just takes the brightest
C   sources without checking for overlap!!
C
C   Inputs:
C      NSRC    I    Number of sources
C      XSRC    R    Array(NSRC) Source X position (degrees)
C      YSRC    R    Array(NSRC) Source Y position (degrees)
C      XSIZ    R    Array(NSRC) Source X size (arcsec)
C      YSIZ    R    Array(NSRC) Source Y size (arcsec)
C      FSRC    R    Array(NSRC) Source flux (Jy)
C      CELLSZ  R    Array(2) Size of cells in map (arcsec)
C      NX      I    Number of X cells in central map field
C      NY      I    Number of Y cells in central map field
C      MAXBOX  I    Maximum number of boxes to create
C   Output:
C      NBOX    I    Number of boxes used
C      XBOX    R    Array(MAXBOX) Box X position (degrees)
C      YBOX    R    Array(MAXBOX) Box Y position (degrees)
C      LBOX    I    Array(MAXBOX) Box size (pixels)
C      FBOX    R    Array(MAXBOX) Box flux (Jy)
C      IWORK   I    Array (MAXBOX) workspace
C      WORK    R    Array (MAXBOX) workspace
C      IERR    I    Error code: 0 => ok
C                           other => error
C-----------------------------------------------------------------------
      INTEGER NSRC, NX, NY, MAXBOX, NBOX, IERR
      REAL XSRC(NSRC), YSRC(NSRC), XSIZ(NSRC), YSIZ(NSRC), FSRC(NSRC),
     *  CELLSZ(2), XBOX(MAXBOX), YBOX(MAXBOX), WORK(NSRC), FBOX(NSRC)
      INTEGER LBOX(MAXBOX), IWORK(NSRC)
C
      REAL XLIM, YLIM
      INTEGER I
      INCLUDE 'INCS:DMSG.INC'
C
      MSGTXT = 'WARNING: Still a dumb version of SCTBX!'
      CALL MSGWRT (1)
C
      NBOX = 0
      DO 10 I=1,NSRC
C                                       Check to see if source is entirely
C                                       within central field
         XLIM = (3600*ABS(XSRC(I)) + 0.5*ABS(XSIZ(I)))/CELLSZ(1)
         YLIM = (3600*ABS(YSRC(I)) + 0.5*ABS(YSIZ(I)))/CELLSZ(2)
         IF (XLIM .GT. 0.5*NX .OR. YLIM .GT. 0.5*NY) THEN
C
C                                       Not in middle of field, add a box
            IF (NBOX.GE.MAXBOX) GO TO 20
            NBOX = NBOX+1
C                                       Convert X,Y from degrees to arcsec
            XBOX(NBOX) = 3600.0*XSRC(I)
            YBOX(NBOX) = 3600.0*YSRC(I)
            FBOX(NBOX) = FSRC(I)
C Box size in pixels
            LBOX(NBOX) = MAX( INT( XSIZ(I) / CELLSZ(1) )+1,
     *                        INT( YSIZ(I) / CELLSZ(2) )+1, 21 )
         ENDIF
10    CONTINUE
C
20    CONTINUE
      IERR = 0
      RETURN
      END
