LOCAL INCLUDE 'INPUT.INC'
C                                       Declarations for inputs
      INTEGER NPARMS
C                                       NPARMS=no. adverbs passed.
C USERID     User number.
C INNAME     Main input file (name).
C INCLASS    Main input file (class).
C INSEQ      Main input file (seq. #).
C INDISK     Disk unit #.       0 => any
C INVERS     Input  file version no.
C NFIELD     Number of input fields.
C SCALR1     Radius for CC select (asec).
C CUTOFF     Minimum flux for CC select.
C SCALR2     Source growing radius (asec).
C ICUT       Min source flux for output.
C NPOINTS    Max sources to print.
C OUTFILE    Output directory name.
C
      PARAMETER (NPARMS=13)
      INTEGER AVTYPE(NPARMS), AVDIM(2,NPARMS)
      CHARACTER AVNAME(NPARMS)*8
LOCAL END

LOCAL INCLUDE 'INPUTDATA.INC'
C                                       DATA statments defining input
C                                       parameters.
C                                       Adverb names
C                      1        2         3          4        5
      DATA AVNAME /'USERID', 'INNAME', 'INCLASS', 'INSEQ', 'INDISK',
C            6         7        8         9         10       11
     *   'INVER',  'NFIELD', 'SCALR1', 'CUTOFF', 'SCALR2',  'ICUT',
C           12        13
     * 'NPOINTS', 'OUTFILE'/
C                                       Adverb data types (PAOOF.INC)
C                      1        2         3          4        5
      DATA AVTYPE /  OOAINT,  OOACAR,   OOACAR,    OOAINT,  OOAINT,
C            6         7        8          9         10       11
     *    OOAINT,    OOAINT,   OOARE,   OOARE,     OOARE,     OOARE,
C           12        13
     *    OOAINT,    OOACAR/ 
C                                       Adverb dimensions (as 2D)
C                      1        2         3          4        5
      DATA AVDIM /   1,1,    12,1,      6,1,       1,1,     1,1,
C           6          7        8         9         10       11
     *    1,1,       1,1,     1,1,      1,1,       1,1,     1,1,
C          12         13
     *    1,1,      48,1/
LOCAL END

      PROGRAM SRCFN
C-----------------------------------------------------------------------
C! Find brightest sources by searching clean components.
C# Utility OOP
C   This software is the subject of a User agreement and is confidential
C   in nature. It shall not be sold or otherwise made available or
C   disclosed to third parties.
C-----------------------------------------------------------------------
      CHARACTER PRGM*6
      INTEGER  IERR, BUFF1(256)
      DATA PRGM /'SRCFN '/
C-----------------------------------------------------------------------
C                                       Startup
      CALL SFNDIN (PRGM, IERR)
C                                       Process tables
      IF (IERR.EQ.0) CALL SFNDAB (IERR)
C                                       Close down files, etc.
 990  CALL DIE (IERR, BUFF1)
C
 999  STOP
      END

      SUBROUTINE SFNDIN (PRGN, IERR)
C-----------------------------------------------------------------------
C   SFNDIN gets input parameters for SRCFN and creates the input object
C   Inputs:
C      PRGN    C*6  Program name
C   Outputs:
C      IERR    I    Error code: 0 => ok
C                               4 => user routine detected error.
C                               5 => catalog troubles
C                               8 => can't start
C-----------------------------------------------------------------------
      CHARACTER PRGN*6
      INTEGER   IERR
C
      INCLUDE 'INCS:DMSG.INC'
      INCLUDE 'INCS:PAOOF.INC'
      INCLUDE 'INPUT.INC'
      INCLUDE 'INPUTDATA.INC'
C-----------------------------------------------------------------------
C                                       Startup,  returns "Input" object
C                                       containing POPS adverbs
      CALL AV2INP (PRGN, NPARMS, AVNAME, AVTYPE, AVDIM, 'Input', IERR)
      IF (IERR.NE.0) GO TO 999
C
 999  RETURN
      END

      SUBROUTINE SFNDAB (IERR)
C-----------------------------------------------------------------------
C   Sum components and print brightest sources.
C   Output:
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      INTEGER   IERR
C
C                                       MAXCC = max. no. CC entries.
C                                       MAXSRC = max. no. sources.
      INTEGER   MAXCC, MAXSRC
      PARAMETER (MAXCC = 10000, MAXSRC = 1000)
C                                       OUTDIR = name of output directory
      CHARACTER*48 OUTDIR
C                                       DEFDIR = default name of directory
      CHARACTER*48 DEFDIR
      PARAMETER (DEFDIR = '/bob1/aips/first/srcfn')
C                                       OUTNAM = name used for output file
      CHARACTER*12 OUTNAM
C
      INTEGER TYPE, DIM(7), NCC, NEWCC, NPNTS, NSRC, LOWCAS,
     *   I, J, K, IWORK(MAXCC)
      REAL RAD1, CUTOF1, RAD2, CUTOF2, XSHIFT, YSHIFT,
     *   XPOS(MAXCC),  YPOS(MAXCC),  FLUX(MAXCC),  WORK(MAXCC),
     *   XSIZ(MAXSRC), YSIZ(MAXSRC), FSRC(MAXSRC), XSRC(MAXSRC),
     *   YSRC(MAXSRC)
      DOUBLE PRECISION RACEN, DECEN, RACC, DECCC,
     *   RA(MAXSRC), DEC(MAXSRC), DWORK(MAXSRC)
      LOGICAL ALLBLK
C                                       NFIELD = number of MX fields
      INTEGER   NFIELD
C                                       Input table and image objects
      CHARACTER*32 INTAB, INIMG
C
      INTEGER   NKEY
C                                       NKEY=no. adverbs to copy to
C                                       INTAB & INIMG
      PARAMETER (NKEY=13)
      CHARACTER INK1(NKEY)*8, OUTK1(NKEY)*32,
     *          INK2(NKEY)*8, OUTK2(NKEY)*32,
     *          CLASS*6
      INCLUDE 'INCS:DMSG.INC'
      INCLUDE 'INCS:PAOOF.INC'
C                                       Adverbs to copy to INTAB
C                      1        2         3          4        5
      DATA INK1   /'USERID', 'INNAME', 'INCLASS', 'INSEQ', 'INDISK',
C           6          7        8         9         10       11
     *   'INVER',  'NFIELD', 'SCALR1', 'CUTOFF',  'SCALR2',  'ICUT',
C          12         13
     * 'NPOINTS', 'OUTFILE'/
C                                       May rename adverbs to INTAB
C                      1        2         3          4        5
      DATA OUTK1  /'USERID',  'NAME',   'CLASS',   'IMSEQ',   'DISK',
C           6          7        8         9         10       11
     *   'VER',    'NFIELD','SCALR1', 'CUTOFF',  'SCALR2',  'ICUT',
C          12         13
     * 'NPOINTS', 'OUTFILE'/
C-----------------------------------------------------------------------
C                                       Copy keywords to INK2, OUTK2
      DO 10 I=1,NKEY
         INK2(I)  = INK1(I)
         OUTK2(I) = OUTK1(I)
  10  CONTINUE
C
C                                       Get number of fields
      CALL OGET ('Input', 'NFIELD', TYPE, DIM, NFIELD, NFIELD, IERR)
      IF (IERR.NE.0) GO TO 999
      IF (NFIELD.LT.1) NFIELD = 1
C                                       Filtering parameters
      CALL OGET ('Input', 'SCALR1', TYPE, DIM, RAD1, RAD1, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET ('Input', 'CUTOFF', TYPE, DIM, CUTOF1, CUTOF1, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET ('Input', 'SCALR2', TYPE, DIM, RAD2, RAD2, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET ('Input', 'ICUT',   TYPE, DIM, CUTOF2, CUTOF2, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Output parameters
      CALL OGET ('Input', 'OUTFILE',TYPE, DIM, OUTDIR, OUTDIR, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET ('Input', 'INNAME',TYPE, DIM, OUTNAM, OUTNAM, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET ('Input', 'NPOINTS',TYPE, DIM, NPNTS, NPNTS, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Get root for class
      CALL OGET ('Input', 'INCLASS', TYPE, DIM, CLASS, CLASS, IERR)
      IF (IERR.NE.0) GO TO 999
C
C                                       Get length of CLASS
      DO 20 J=LEN(CLASS),1,-1
            IF (CLASS(J:J).NE.' ') GO TO 30
  20  CONTINUE
      J = 0
  30  CONTINUE
C
C                                       Make sure there is room for number.
      IF (NFIELD.GT.1 .AND. J.GT.LEN(CLASS)-2) THEN
         WRITE(MSGTXT,1000) CLASS(1:J)
1000     FORMAT('INCLASS "',A,'" does not have room for field number.')
         CALL MSGWRT(8)
         GO TO 999
      ENDIF
C
C                                       Loop over fields
      NCC = 0
      DO 50 I=1,NFIELD
         IF (I.GT.1) WRITE(CLASS(J+1:),1001) I-1
1001     FORMAT(I2.2)
         WRITE(INTAB,1002) 'Input table',I
         WRITE(INIMG,1002) 'Input image',I
1002     FORMAT(A,I3)
C
C                                       Create input table object
         CALL CREATE (INTAB, 'TABLE', IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Copy adverbs to table object
         CALL IN2OBJ ('Input', NKEY, INK1, OUTK1, INTAB, IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Set table type
         DIM(1) = 2
         DIM(2) = 1
         DIM(3) = 0
         CALL OPUT (INTAB, 'TBLTYPE', OOACAR, DIM, 'CC', 'CC',
     *      IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Set CLASS
         DIM(1) = LEN(CLASS)
         DIM(2) = 1
         DIM(3) = 0
         CALL OPUT (INTAB, 'CLASS', OOACAR, DIM, CLASS, CLASS,
     *      IERR)
         IF (IERR.NE.0) GO TO 999
C
C                                       Create input image descriptor
         CALL CREATE (INIMG, 'IMAGE', IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Copy adverbs to image descriptor
         CALL IN2OBJ ('Input', NKEY, INK2, OUTK2, INIMG, IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Set CLASS
         DIM(1) = LEN(CLASS)
         DIM(2) = 1
         DIM(3) = 0
         CALL OPUT (INIMG, 'CLASS', OOACAR, DIM, CLASS, CLASS, IERR)
         IF (IERR.NE.0) GO TO 999
C
C                                       Now process this image
         IF (NFIELD.GT.1) THEN
            WRITE(MSGTXT, 1003) I,NFIELD
 1003       FORMAT ('Reading image ', I8, ' of ', I8)
            CALL MSGWRT (1)
         ENDIF
C                                       Open input image
         CALL OOPEN (INIMG, 'READ', IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Get RA, Dec for phase center
         CALL GETPOS(INIMG, RACEN, DECEN, XSHIFT, YSHIFT, IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Close input image
         CALL OCLOSE (INIMG, IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Open input table
         CALL OOPEN (INTAB, 'READ', IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Read clean components
C                                       and add to list
         CALL READCC (INTAB, MAXCC-NCC, NEWCC,
     *      XPOS(NCC+1), YPOS(NCC+1), FLUX(NCC+1),
     *      WORK, IWORK, IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Close Input table
         CALL OCLOSE (INTAB, IERR)
         IF (IERR.NE.0) GO TO 999
C                                       Destroy table, image objects
         CALL DESTRY (INTAB, IERR)
         IF (IERR.NE.0) GO TO 999
         CALL DESTRY (INIMG, IERR)
         IF (IERR.NE.0) GO TO 999
C
C                                       Convert CC positions to X,Y
C                                       related to field center
C
C Note that we're assuming here that RACEN and DECEN are identical
C for all fields!  We do things in this order so that clean components
C get properly grouped even if the subfields overlap.
C
         DO 40 K=NCC+1,NCC+NEWCC
            CALL XY2SKY(XPOS(K),YPOS(K),RACEN,DECEN,XSHIFT,YSHIFT,
     *         RACC,DECCC,IERR)
            IF (IERR .NE. 0) GO TO 999
            CALL SKY2XY(RACC,DECCC,RACEN,DECEN,XPOS(K),YPOS(K),IERR)
            IF (IERR .NE. 0) GO TO 999
  40     CONTINUE
         NCC = NCC+NEWCC
  50  CONTINUE
C
C                                       Remove isolated clean components
      CALL CCREMV (XPOS, YPOS, FLUX, WORK, NCC, RAD1, CUTOF1, IERR)
      IF (IERR.NE.0) GO TO 999
C
C                                       Group clean components
      CALL CCGRP (XPOS, YPOS, FLUX, NCC, RAD2, IWORK, MAXSRC,
     *   XSRC, YSRC, XSIZ, YSIZ, FSRC, NSRC, IERR)
      IF (IERR.NE.0) GO TO 999
C
C                                       Convert X,Y to RA,DEC
      DO 60 K=1,NSRC
         CALL XY2SKY(XSRC(K),YSRC(K),RACEN,DECEN,0.0,0.0,
     *      RA(K),DEC(K),IERR)
         IF (IERR .NE. 0) GO TO 999
  60  CONTINUE
C
C                                       Sort by decreasing flux
      CALL SORT5 (NSRC, FSRC, XSIZ, YSIZ, RA, DEC, DWORK, IWORK, IERR)
      IF (IERR.NE.0) GO TO 999
C
C                                       Convert output directory to lowercase
      ALLBLK = .TRUE.
      LOWCAS = ICHAR('a') - ICHAR('A')
      DO 70 I=1,LEN(OUTDIR)
         IF (OUTDIR(I:I).GE.'A' .AND. OUTDIR(I:I).LE.'Z') THEN
            OUTDIR(I:I) = CHAR( ICHAR(OUTDIR(I:I)) + LOWCAS )
         END IF
         IF (OUTDIR(I:I).NE.' ') ALLBLK = .FALSE.
  70  CONTINUE
C                                       If OUTDIR blank, use default
      IF (ALLBLK) OUTDIR = DEFDIR
C                                       Convert OUTNAM to lowercase too
      DO 80 I=1,LEN(OUTNAM)
         IF (OUTNAM(I:I).GE.'A' .AND. OUTNAM(I:I).LE.'Z') THEN
            OUTNAM(I:I) = CHAR( ICHAR(OUTNAM(I:I)) + LOWCAS )
         END IF
  80  CONTINUE
C                                       Print sources
C
      CALL WRTSRC (OUTDIR, OUTNAM, RA, DEC, XSIZ, YSIZ, FSRC, NSRC,
     *   CUTOF2, NPNTS, IERR)
      IF (IERR.NE.0) GO TO 999
C
 999  RETURN
C-----------------------------------------------------------------------
      END

      SUBROUTINE CCREMV (XPOS, YPOS, FLUX, SFLUX, NCC, RADIUS, CUTOFF,
     *   IERR)
C-----------------------------------------------------------------------
C   Remove isolated weak clean components from list.
C   Inputs:
C      XPOS    R    CC X position (degrees)
C      YPOS    R    CC Y position (degrees)
C      FLUX    R    CC flux
C      NCC     I    Number of clean components
C      RADIUS  R    Radius for combining components (arcsec)
C      MAXSRC  I    Maximum number of sources
C   Outputs:
C      SFLUX   R    Used as workspace
C      XPOS    D    X,Y,F are modified to remove weak components
C      YPOS    D
C      FLUX    R
C      NCC     I    Number of clean components is modified too
C      IERR    I    Return error code, 0=>OK
C-----------------------------------------------------------------------
      INTEGER NCC, IERR
      REAL RADIUS, CUTOFF, FLUX(NCC), SFLUX(NCC), XPOS(NCC), YPOS(NCC)
C
C
C
      INTEGER I, J
      REAL MXDIS, SUM, DIS
      INCLUDE 'INCS:DMSG.INC'
C
C                                       Sum flux near each component
      MXDIS = (RADIUS / 3600.0) ** 2
      DO 200 I = 1,NCC
         SUM = 0.0
         DO 100 J = 1,NCC
            DIS = (XPOS(I)-XPOS(J)) ** 2 + (YPOS(I)-YPOS(J)) ** 2
            IF (DIS.LE.MXDIS) SUM = SUM + FLUX(J)
 100     CONTINUE
         SFLUX(I) = SUM
 200  CONTINUE
C
C                                       Remove weak isolated components
      J = 0
      DO 300 I = 1,NCC
         IF (SFLUX(I) .GT. CUTOFF) THEN
            J = J + 1
            XPOS(J) = XPOS(I)
            YPOS(J) = YPOS(I)
            FLUX(J) = FLUX(I)
            SFLUX(J) = SFLUX(I)
         END IF
 300  CONTINUE
      IF (J.NE.NCC) THEN
         WRITE (MSGTXT,1000) NCC, J
1000     FORMAT('Eliminating weak isolated CCs reduced CC from',
     *      I7,' to',I7)
         CALL MSGWRT (1)
      ENDIF
      NCC = J
      IERR = 0
      RETURN
      END

      SUBROUTINE CCGRP (XPOS, YPOS, FLUX, NCC, RADIUS, IWORK, MAXSRC,
     *   XSRC, YSRC, XSIZ, YSIZ, FSRC, NSRC, IERR)
C-----------------------------------------------------------------------
C   Group clean components into "sources".  Returns flux and centroid
C   of each source.
C   Inputs:
C      XPOS    R    CC X position (degrees)
C      YPOS    R    CC Y position (degrees)
C      FLUX    R    CC flux
C      NCC     I    Number of clean components
C      RADIUS  R    Radius for combining components (arcsec)
C      MAXSRC  I    Maximum number of sources
C   Outputs:
C      IWORK   I    Used as workspace
C      XSRC    R    Source X position (degrees)
C      YSRC    R    Source Y position (degrees)
C      XSIZ    R    Source X size (degrees)
C      YSIZ    R    Source Y size (degrees)
C      FSRC    R    Source flux
C      NSRC    I    Number of sources
C      IERR    I    Return error code, 0=>OK
C-----------------------------------------------------------------------
      INTEGER NCC, MAXSRC, NSRC, IERR
      INTEGER IWORK(NCC)
      REAL RADIUS, FLUX(NCC), XSIZ(MAXSRC), YSIZ(MAXSRC), FSRC(MAXSRC)
      REAL XPOS(NCC), YPOS(NCC), XSRC(MAXSRC), YSRC(MAXSRC)
C
C
C
      INTEGER I, J, K, IGROUP, OGROUP, JSRC
      REAL MXDIS, SUM, DIS
      REAL XMIN, XMAX, YMIN, YMAX
      INCLUDE 'INCS:DMSG.INC'
C
C                                       Assign initial group number 0
      DO 100 I=1,NCC
         IWORK(I) = 0
100   CONTINUE
C
C                                       Group components using RADIUS
      MXDIS = (RADIUS / 3600.0) ** 2
      IGROUP = 0
      NSRC = 0
      DO 400 I=1,NCC
C                                       Create new group if not in group
         IF (IWORK(I).EQ.0) THEN
            IGROUP = IGROUP+1
            NSRC = NSRC+1
            IWORK(I) = IGROUP
         ENDIF
C                                       OGROUP is current group number
         OGROUP = IWORK(I)
         DO 300 J=I+1,NCC
C                                       Check distance if not in same group
            IF (OGROUP.NE.IWORK(J)) THEN
               DIS = (XPOS(I)-XPOS(J)) ** 2 + (YPOS(I)-YPOS(J)) ** 2
               IF (DIS.LE.MXDIS) THEN
                  IF (IWORK(J).EQ.0) THEN
C                                       Add isolated point to this group
                     IWORK(J) = OGROUP
                  ELSE
C                                       Merge groups
                     DO 200 K=1,NCC
                        IF(IWORK(K).EQ.OGROUP) IWORK(K) = IWORK(J)
200                  CONTINUE
                     OGROUP = IWORK(J)
                     NSRC = NSRC-1
                  ENDIF
               ENDIF
            ENDIF
300      CONTINUE
400   CONTINUE
C
      IF (NSRC.GT.MAXSRC) THEN
         WRITE (MSGTXT,1000) NSRC, MAXSRC
         CALL MSGWRT (8)
         IERR = 5
         GO TO 999
      ENDIF
C
C Now we know that there are NSRC different groups, with numbers in
C the range 1 to IGROUP.  Compute fluxes, sizes for each group.
C
      JSRC = 0
      DO 600 OGROUP=1,IGROUP
         SUM = 0.0
         XMIN =  1.E20
         XMAX = -1.E20
         YMIN =  1.E20
         YMAX = -1.E20
         K = 0
         DO 500 I=1,NCC
            IF (IWORK(I).EQ.OGROUP) THEN
               SUM = SUM+FLUX(I)
               XMIN = MIN(XMIN,XPOS(I))
               XMAX = MAX(XMAX,XPOS(I))
               YMIN = MIN(YMIN,YPOS(I))
               YMAX = MAX(YMAX,YPOS(I))
               K = K+1
            ENDIF
500      CONTINUE
C                                       Increment JSRC if any sources in group
         IF (K.GT.0) THEN
            JSRC = JSRC+1
            FSRC(JSRC) = SUM
            XSRC(JSRC) = 0.5*(XMIN+XMAX)
            YSRC(JSRC) = 0.5*(YMIN+YMAX)
            XSIZ(JSRC) = (XMAX-XMIN)
            YSIZ(JSRC) = (YMAX-YMIN)
         ENDIF
600   CONTINUE
C
C                                       Consistency check
      IF (JSRC.NE.NSRC) THEN
         WRITE (MSGTXT,1001) JSRC, NSRC
         CALL MSGWRT (8)
         IERR = 5
         GO TO 999
      ENDIF
      WRITE(MSGTXT,1002) NSRC
      CALL MSGWRT (1)
      IERR = 0
C
 999  RETURN
C-----------------------------------------------------------------------
 1000 FORMAT ('TOO MANY SOURCES FOR INTERNAL ARRAYS:', I7, '>', I8)
 1001 FORMAT ('CCGRP: Program bug!! JSRC=', I7, ', NSRC=', I8)
 1002 FORMAT ('Found ',I8,' different sources')
      END

      SUBROUTINE WRTSRC (OUTDIR, OUTNAM, RA, DEC, XSIZ, YSIZ, FSRC,
     *   NSRC, CUTOFF, NPNTS, IERR)
C-----------------------------------------------------------------------
C   Write positions of sources to output file.
C   Inputs:
C      OUTDIR  C    Output file directory
C      OUTNAM  C    Output file name
C      RA      D    Source Right Ascension (degrees)
C      DEC     D    Source Declination (degrees)
C      XSIZ    R    Source X size (degrees)
C      YSIZ    R    Source Y size (degrees)
C      FSRC    R    Source flux.  Sources must be sorted by
C                   decreasing flux.
C      NSRC    I    Number of sources
C      CUTOFF  R    Minimum flux to write to file
C      NPNTS   I    Maximum number of sources to write to file
C   Outputs:
C      IERR    I    Return error code, 0=>OK
C-----------------------------------------------------------------------
C
      CHARACTER*(*) OUTDIR, OUTNAM
      INTEGER NSRC, NPNTS, IERR
      DOUBLE PRECISION RA(NSRC), DEC(NSRC)
      REAL XSIZ(NSRC), YSIZ(NSRC), FSRC(NSRC), CUTOFF
C
      CHARACTER*61 OUTFIL
      INTEGER I, J, IMAX, LOUT, RAHH, RAMM, DCDD, DCMM
      REAL RASS, DCSS
      CHARACTER*1 DCSIGN
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C                                       Build the filename
      OUTFIL = OUTDIR
      DO 100 I=LEN(OUTDIR),1,-1
         IF (OUTDIR(I:I).NE.' ') THEN
            J = I+1
            GO TO 200
         ENDIF
 100  CONTINUE
 200  CONTINUE
C                                       Add filename after directory
      OUTFIL(J:) = '/' // OUTNAM
C                                       Find length of OUTFIL
      DO 300 I=LEN(OUTFIL),1,-1
         IF (OUTFIL(I:I).NE.' ') THEN
            LOUT = I
            GO TO 400
         ENDIF
 300  CONTINUE
 400  CONTINUE
C                                       Replace embedded blanks
      DO 500 I=J,LOUT
         IF (OUTFIL(I:I).EQ.' ') OUTFIL(I:I) = '_'
 500  CONTINUE
C                                       Write the file
      OPEN (UNIT=55, FILE=OUTFIL, STATUS='UNKNOWN', ERR=799)
      IF (NSRC .LT. NPNTS) THEN
         IMAX = NSRC
      ELSE
         IMAX = NPNTS
      END IF
      DO 600 I = 1,IMAX
         IF (FSRC(I) .LT. CUTOFF) THEN
            J = I-1
            GO TO 700
         ENDIF
C                                       Convert RA degrees to HMS
C                                       (3 is number of seconds digits)
         CALL RA2HMS(RA(I),3,RAHH,RAMM,RASS,IERR)
         IF (IERR .NE. 0) GO TO 999
C                                       Convert Dec degrees to DMS
C                                       (2 is number of seconds digits)
         CALL DC2DMS(DEC(I),2,DCSIGN,DCDD,DCMM,DCSS,IERR)
         IF (IERR .NE. 0) GO TO 999
C                                       Write position to the file
         WRITE(55,1001,ERR=899) RAHH, RAMM, RASS,
     *      DCSIGN, DCDD, DCMM, DCSS,
     *      3600.*XSIZ(I), 3600.*YSIZ(I), FSRC(I)
 600     CONTINUE
C
      J = IMAX
 700  CONTINUE
      CLOSE (UNIT=55)
      WRITE (MSGTXT,1000) J, OUTFIL(1:LOUT)
      CALL MSGWRT (1)
      IERR = 0
      GO TO 999
C
 799  CONTINUE
      WRITE (MSGTXT,1002) OUTFIL(1:LOUT)
      CALL MSGWRT (8)
      IERR = 5
      GO TO 999
C
 899  CONTINUE
      WRITE (MSGTXT,1003) OUTFIL(1:LOUT)
      CALL MSGWRT (8)
      IERR = 5
      GO TO 999
C
 999  RETURN
C
 1000 FORMAT('Wrote',I7,' sources to file ',A)
 1001 FORMAT (1X, I2.2, I3.2, F7.3, 1X, A1, I2.2, I3.2, F6.2, 2X,
     *   2F10.3, F13.5)
 1002 FORMAT ('ERROR OPENING OUTPUT FILE ',A)
 1003 FORMAT ('ERROR WRITING TO OUTPUT FILE ',A)
      END
