C  Green Bank 1.4 GHz catalog routines, 5 May 1993
C-----------------------------------------------------------------------
      SUBROUTINE CTOPN (UNIT, FILNAM, IERR)
C-----------------------------------------------------------------------
C   Open Green Bank 1.4 GHz catalog.
C
C   Inputs:
C      UNIT     I     Unit number of catalog.
C      FILNAM   C*(*) Name of catalog (including pathname).  If FILNAM
C                     is blank, uses default name:
C                        '/bob1/aips/first/north20/first20.bin'
C   Outputs:
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER UNIT, IERR
      CHARACTER*(*) FILNAM
C
      CHARACTER*(*) DFAULT
      PARAMETER (DFAULT = '/bob1/aips/first/north20/first20.bin')
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
      IF (FILNAM .EQ. ' ') THEN
         OPEN (UNIT=UNIT, FILE=DFAULT, STATUS='OLD',
     *      FORM='UNFORMATTED', RECL=36, ACCESS='DIRECT', ERR=199)
      ELSE
         OPEN (UNIT=UNIT, FILE=FILNAM, STATUS='OLD',
     *      FORM='UNFORMATTED', RECL=36, ACCESS='DIRECT', ERR=199)
      ENDIF
      IERR = 0
      RETURN
C
C Error exit
C
 199  CONTINUE
      MSGTXT = 'Error opening 20cm Green Bank catalog'
      CALL MSGWRT (8)
      IF (FILNAM .EQ. ' ') THEN
         MSGTXT = DFAULT
      ELSE
         MSGTXT = FILNAM
      ENDIF
      CALL MSGWRT (8)
      IERR = 1
      RETURN
      END

      SUBROUTINE CTCLS (UNIT, IERR)
C-----------------------------------------------------------------------
C   Close catalog.
C
C   Inputs:
C      UNIT     I     Unit number of catalog.
C   Outputs:
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER UNIT, IERR
C
C-----------------------------------------------------------------------
C
      CLOSE (UNIT=UNIT)
      IERR = 0
      RETURN
      END

      SUBROUTINE CTGET (UNIT, IRECNO, NAME, RA, DEC, POSERR,
     *   F20, F6, F80, IERR)
C-----------------------------------------------------------------------
C   Read one source from catalog by record number.
C
C   Inputs:
C      UNIT     I     Unit number of (already opened) catalog.
C      IRECNO   I     Number of record in file (first record is number 1).
C   Outputs:
C      NAME     C*12  The name of the object.
C      RA       D     RA of object (degrees, J2000).
C      DEC      D     Dec of object (degrees, J2000).
C      POSERR   R     Uncertainty in object position (degrees).
C      F20      R     20cm flux (Jy).
C      F6       R     6cm flux (Jy). <0 if no 6cm detection.
C      F80      R     80cm flux (Jy). <0 if no 80cm detection.
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER          UNIT, IRECNO, IERR
      REAL             POSERR, F20, F6, F80
      DOUBLE PRECISION RA, DEC
      CHARACTER*12     NAME
C
      REAL             RECORD(9)
      CHARACTER*12     RNAME
      EQUIVALENCE      (RNAME, RECORD(7))
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
      READ(UNIT,REC=IRECNO,ERR=899) RECORD
      RA     = RECORD(1)
      DEC    = RECORD(2)
      POSERR = RECORD(3)
      F20    = RECORD(4)
      F6     = RECORD(5)
      F80    = RECORD(6)
      NAME   = RNAME
      IERR = 0
      RETURN
C
C Error in read
C
 899  CONTINUE
      WRITE(MSGTXT,1001) IRECNO
1001  FORMAT('Error reading record',I8,' from 20cm Green Bank catalog')
      CALL MSGWRT (8)
      IERR = 1
      RETURN
      END

      SUBROUTINE CTFND (UNIT, RA0, DEC0, DIST, IRECNO, NAME,
     *   RA, DEC, POSERR, F20, F6, F80, SEP, IERR)
C-----------------------------------------------------------------------
C   Find first source in catalog that is within DIST degrees of position
C   RA0, DEC0.
C
C   Inputs:
C      UNIT     I     Unit number of (already opened) catalog.
C      RA0      D     Reference RA (J2000, degrees).
C      DEC0     D     Reference Declination (J2000, degrees).
C      DIST     R     Radius of search circle around RA0, DEC0 (degrees).
C   Outputs:
C      IRECNO   I     Number of record in file (first record is number 1).
C                     <= 0 if no source is found.
C      NAME     C*12  The name of the object.
C      RA       D     RA of object (degrees, J2000).
C      DEC      D     Dec of object (degrees, J2000).
C      POSERR   R     Uncertainty in object position (degrees).
C      F20      R     20cm flux (Jy).
C      F6       R     6cm flux (Jy). <0 if no 6cm detection.
C      F80      R     80cm flux (Jy). <0 if no 80cm detection.
C      SEP      R     Actual separation between RA0,DEC0 and RA,DEC (degrees)
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER          UNIT, IRECNO, IERR
      REAL             DIST, POSERR, F20, F6, F80, SEP
      DOUBLE PRECISION RA0, DEC0, RA, DEC
      CHARACTER*12     NAME
C
      INTEGER CATMAX, ILO, IHI, IMID
      PARAMETER (CATMAX = 78388)
      DOUBLE PRECISION DECLIM
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
C Search to find first source that has Dec < Dec0 + DIST.
C Note that catalog is sorted in order of descending declination.
C
      DECLIM = DEC0+DIST
C
C Use binary search
C
      ILO = 0
      IHI = CATMAX
  10  CONTINUE
         IMID = (IHI+ILO)/2
C
C Read this source
C
         CALL CTGET (UNIT, IMID, NAME, RA, DEC, POSERR,
     *      F20, F6, F80, IERR)
         IF (IERR .NE. 0) RETURN
C
         IF (DEC.GT.DECLIM) THEN
            ILO = IMID
         ELSE
            IHI = IMID
         ENDIF
         IF (IHI.GT.ILO+1) GO TO 10
C
C OK, we know DECLIM is at IHI: make sure it is not outside
C range of catalog
C
      CALL CTGET (UNIT, IHI, NAME, RA, DEC, POSERR,
     *   F20, F6, F80, IERR)
      IF (IERR .NE. 0) RETURN
C
      IF (DEC.LT.DEC0-DIST .OR. DEC.GT.DEC0+DIST) THEN
C
C No sources in range DEC0 +/- DIST
C
         IRECNO = -1
         WRITE(MSGTXT,1000) DEC0
1000     FORMAT('Warning: declination',F10.5,
     *      ' is outside dec range of 20cm catalog')
         CALL MSGWRT(1)
      ELSE
C
C Find next source within distance DIST starting at IHI-1
C
         IRECNO = IHI-1
         CALL CTNXT (UNIT, RA0, DEC0, DIST, IRECNO, NAME,
     *      RA, DEC, POSERR, F20, F6, F80, SEP, IERR)
      ENDIF
C
      IERR = 0
      RETURN
      END

      SUBROUTINE CTNXT (UNIT, RA0, DEC0, DIST, IRECNO, NAME,
     *   RA, DEC, POSERR, F20, F6, F80, SEP, IERR)
C-----------------------------------------------------------------------
C   Find next source in catalog that is within DIST degrees of position
C   RA0, DEC0.
C
C   Inputs:
C      UNIT     I     Unit number of (already opened) catalog.
C      RA0      D     Reference RA (J2000, degrees).
C      DEC0     D     Reference Declination (J2000, degrees).
C      DIST     R     Radius of search circle around RA0, DEC0 (degrees).
C      IRECNO   I     Number of record in file where last source was
C                     found (first record is number 1).
C   Outputs:
C      IRECNO   I     Changed to indicate number of next record.
C                     <= 0 if no source is found.
C      NAME     C*12  The name of the object.
C      RA       D     RA of object (degrees, J2000).
C      DEC      D     Dec of object (degrees, J2000).
C      POSERR   R     Uncertainty in object position (degrees).
C      F20      R     20cm flux (Jy).
C      F6       R     6cm flux (Jy). <0 if no 6cm detection.
C      F80      R     80cm flux (Jy). <0 if no 80cm detection.
C      SEP      R     Actual separation between RA0,DEC0 and RA,DEC (degrees)
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER          UNIT, IRECNO, IERR
      REAL             DIST, POSERR, F20, F6, F80, SEP
      DOUBLE PRECISION RA0, DEC0, RA, DEC
      CHARACTER*12     NAME
C
      REAL GCIRCL
      INTEGER CATMAX, I
      PARAMETER (CATMAX = 78388)
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
C Just check each record consecutively starting with IRECNO+1
C
      IF (IRECNO .LT. 0) THEN
         MSGTXT = 'Error in CATNXT: record number <= 0'
         CALL MSGWRT (8)
         IERR = 1
         RETURN
      ENDIF
C
      DO 10 I=IRECNO+1, CATMAX
         CALL CTGET (UNIT, I, NAME, RA, DEC, POSERR,
     *      F20, F6, F80, IERR)
         IF (IERR.NE.0) RETURN
C See if we've gone past last match
         IF (DEC0-DIST .GT. DEC) GO TO 20
C Check great circle distance
         SEP = GCIRCL(RA,DEC,RA0,DEC0)
         IF (SEP.LE.DIST) THEN
C Found one
             IRECNO = I
             RETURN
         ENDIF
  10  CONTINUE
C
C No more matching records, return -1 for IRECNO
C
  20  CONTINUE
      IRECNO = -1
      RETURN
      END

      SUBROUTINE BRTFND (UNIT, RA0, DEC0, DIST, CUTOFF, FREQ,
     *   IRECNO, NAME, RA, DEC, POSERR, F20, F6, F80, PBF20, SEP, IERR)
C-----------------------------------------------------------------------
C   Find first source in catalog that is within DIST degrees of position
C   RA0, DEC0 and that appears brighter than CUTOFF Jy when attenuated
C   by the primary beam.
C
C   Inputs:
C      UNIT     I     Unit number of (already opened) catalog.
C      RA0      D     Reference RA (J2000, degrees).
C      DEC0     D     Reference Declination (J2000, degrees).
C      DIST     R     Radius of search circle around RA0, DEC0 (degrees).
C      CUTOFF   R     Minimum measured flux (Jy, modified by primary beam).
C      FREQ     R     Frequency of observation (Hz).
C   Outputs:
C      IRECNO   I     Number of record in file (first record is number 1).
C                     <= 0 if no source is found.
C      NAME     C*12  The name of the object.
C      RA       D     RA of object (degrees, J2000).
C      DEC      D     Dec of object (degrees, J2000).
C      POSERR   R     Uncertainty in object position (degrees).
C      F20      R     True 20cm flux (Jy).
C      F6       R     6cm flux (Jy). <0 if no 6cm detection.
C      F80      R     80cm flux (Jy). <0 if no 80cm detection.
C      PBF20    R     20cm flux (Jy) attenuated by primary beam response.
C      SEP      R     Actual separation between RA0,DEC0 and RA,DEC (degrees)
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER          UNIT, IRECNO, IERR
      REAL             DIST, CUTOFF, FREQ, POSERR,
     *                 F20, F6, F80, PBF20, SEP
      DOUBLE PRECISION RA0, DEC0, RA, DEC
      CHARACTER*12     NAME
C
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
C Search to find first source that has Dec < Dec0 + DIST.
C
      CALL CTFND (UNIT, RA0, DEC0, DIST, IRECNO, NAME,
     *   RA, DEC, POSERR, F20, F6, F80, SEP, IERR)
      IF (IERR.NE.0 .OR. IRECNO.LE.0) RETURN
C
C Now search for source that satisfies flux criterion
C 
      IRECNO = IRECNO-1
      CALL BRTNXT (UNIT, RA0, DEC0, DIST, CUTOFF, FREQ,
     *   IRECNO, NAME, RA, DEC, POSERR, F20, F6, F80, PBF20, SEP, IERR)
      IF (IERR.NE.0) RETURN
C
      RETURN
      END

      SUBROUTINE BRTNXT (UNIT, RA0, DEC0, DIST, CUTOFF, FREQ,
     *   IRECNO, NAME, RA, DEC, POSERR, F20, F6, F80, PBF20, SEP, IERR)
C-----------------------------------------------------------------------
C   Find next source in catalog that is within DIST degrees of position
C   RA0, DEC0 and that appears brighter than CUTOFF Jy when attenuated
C   by the primary beam.
C
C   Inputs:
C      UNIT     I     Unit number of (already opened) catalog.
C      RA0      D     Reference RA (J2000, degrees).
C      DEC0     D     Reference Declination (J2000, degrees).
C      DIST     R     Radius of search circle around RA0, DEC0 (degrees).
C      CUTOFF   R     Minimum measured flux (Jy, modified by primary beam).
C      FREQ     R     Frequency of observation (Hz).
C      IRECNO   I     Number of record in file where last source was
C                     found (first record is number 1).
C   Outputs:
C      IRECNO   I     Changed to indicate number of next record.
C                     <= 0 if no source is found.
C      NAME     C*12  The name of the object.
C      RA       D     RA of object (degrees, J2000).
C      DEC      D     Dec of object (degrees, J2000).
C      POSERR   R     Uncertainty in object position (degrees).
C      F20      R     True 20cm flux (Jy).
C      F6       R     6cm flux (Jy). <0 if no 6cm detection.
C      F80      R     80cm flux (Jy). <0 if no 80cm detection.
C      PBF20    R     20cm flux (Jy) attenuated by primary beam response.
C      SEP      R     Actual separation between RA0,DEC0 and RA,DEC (degrees)
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER          UNIT, IRECNO, IERR
      REAL             DIST, CUTOFF, FREQ, POSERR,
     *                 F20, F6, F80, PBF20, SEP
      DOUBLE PRECISION RA0, DEC0, RA, DEC
      CHARACTER*12     NAME
C
      REAL PB, SBEAM
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
C Get next source
C
      CALL CTNXT (UNIT, RA0, DEC0, DIST, IRECNO, NAME,
     *   RA, DEC, POSERR, F20, F6, F80, SEP, IERR)
      IF (IERR.NE.0) RETURN
C
C Loop until we find source that satisfies flux criterion or until we
C run out of sources.
C 
  10  CONTINUE
         IF (IRECNO.LE.0) RETURN
C
         PB = SBEAM(SEP, FREQ, IERR)
         IF (IERR.NE.0) RETURN
C
         PBF20 = PB*F20
         IF (PBF20.GE.CUTOFF) RETURN
C
         CALL CTNXT (UNIT, RA0, DEC0, DIST, IRECNO, NAME,
     *      RA, DEC, POSERR, F20, F6, F80, SEP, IERR)
         IF (IERR.NE.0) RETURN
C
      GO TO 10
      END
