C  Field list routines, ASCII catalog version, 27 May 1994
C-----------------------------------------------------------------------
LOCAL INCLUDE 'FIELDS.INC'
C
C FLDMAX is number of records in field catalog
C RAEDGE and DCEDGE are RA and Dec values at grid of pixel positions
C spaced all the way around along the edges of the field
C
      INTEGER FLDMAX, NEDGE
      PARAMETER (NEDGE=40)
      DOUBLE PRECISION RAEDGE(NEDGE), DCEDGE(NEDGE)
      COMMON /FLDCTX/ RAEDGE, DCEDGE
      COMMON /FLDCTY/ FLDMAX
LOCAL END
C-----------------------------------------------------------------------
      SUBROUTINE FLDOPN (UNIT, FILNAM, IERR)
C-----------------------------------------------------------------------
C   Open ascii field catalog
C
C   Inputs:
C      UNIT     I     Unit number of catalog.
C      FILNAM   C*(*) Name of catalog (including pathname).  If FILNAM
C                     is blank, uses default name:
C                        '/bob1/aips/first/fields/fields.cat'
C   Outputs:
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER UNIT, IERR
      CHARACTER*(*) FILNAM
C
      CHARACTER*(*) DFAULT
      PARAMETER (DFAULT = '/bob1/aips/first/fields/fields.cat')
      INTEGER FILELN
      INCLUDE 'FIELDS.INC'
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
      IF (FILNAM .EQ. ' ') THEN
         OPEN (UNIT=UNIT, FILE=DFAULT, STATUS='OLD',
     *      FORM='FORMATTED', RECL=32, ACCESS='DIRECT', ERR=199)
      ELSE
         OPEN (UNIT=UNIT, FILE=FILNAM, STATUS='OLD',
     *      FORM='FORMATTED', RECL=32, ACCESS='DIRECT', ERR=199)
      ENDIF
C
C Save length of file in records in common
C
      FLDMAX = FILELN(UNIT)
      IF (FLDMAX.LT.0) THEN
         MSGTXT = 'Unable to get length of file using FILELN'
         CALL MSGWRT (8)
         GO TO 199
      ENDIF
      IF (MOD(FLDMAX,32) .NE. 0) THEN
         MSGTXT = 'File length is not a multiple of 32 bytes'
         CALL MSGWRT (8)
         GO TO 199
      ENDIF
      FLDMAX = FLDMAX / 32
      IERR = 0
      RETURN
C
C Error exit
C
 199  CONTINUE
      MSGTXT = 'Error opening file with field list'
      CALL MSGWRT (8)
      IF (FILNAM .EQ. ' ') THEN
         MSGTXT = DFAULT
      ELSE
         MSGTXT = FILNAM
      ENDIF
      CALL MSGWRT (8)
      IERR = 1
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE FLDCLS (UNIT, IERR)
C-----------------------------------------------------------------------
C   Close catalog.
C
C   Inputs:
C      UNIT     I     Unit number of catalog.
C   Outputs:
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER UNIT, IERR
C
C-----------------------------------------------------------------------
C
      CLOSE (UNIT=UNIT)
      IERR = 0
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE FLDGET (UNIT, IRECNO, NAME, RA, DEC, IERR)
C-----------------------------------------------------------------------
C   Read one field from catalog by record number.
C
C   Inputs:
C      UNIT     I     Unit number of (already opened) catalog.
C      IRECNO   I     Number of record in file (first record is number 1).
C   Outputs:
C      NAME     C*12  The name of the object.
C      RA       D     RA of field center (degrees, J2000).
C      DEC      D     Dec of field center (degrees, J2000).
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER          UNIT, IRECNO, IERR
      DOUBLE PRECISION RA, DEC
      CHARACTER*12     NAME
C
      INTEGER     RAH, RAM, RAS, DECD, DECM, DECS
      CHARACTER*1 DECSGN
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
      READ(UNIT,REC=IRECNO,ERR=899,FMT=1) NAME, RAH, RAM, RAS,
     *   DECSGN, DECD, DECM, DECS
   1  FORMAT(A12,1X,I2,1X,I2,1X,I2,1X,A1,I2,1X,I2,1X,I2)
C
C Convert RA, DEC to degrees
C
      RA  = 15.D0*( RAH  + (RAM  + RAS /60.D0)/60.D0 )
      DEC = DECD + (DECM + DECS/60.D0)/60.D0
      IF (DECSGN.EQ.'-') DEC = -DEC
      IERR = 0
      RETURN
C
C Error in read
C
 899  CONTINUE
      WRITE(MSGTXT,1001) IRECNO
1001  FORMAT('Error reading record',I8,' from field center catalog')
      CALL MSGWRT (8)
      IERR = 1
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE FLDFND (UNIT, RA0, DEC0, NX, NY, DX, DY, DIST, IRECNO,
     *   NAME, RA, DEC, IERR)
C-----------------------------------------------------------------------
C   Find first field in catalog that is within DIST degrees of the edge
C   of field defined by center (RA0,DEC0), dimension (NX,NY),
C   and pixel scale (DX,DY).
C
C   Inputs:
C      UNIT     I     Unit number of (already opened) catalog.
C      RA0      D     Reference RA (J2000, degrees).
C      DEC0     D     Reference Declination (J2000, degrees).
C      NX, NY   I     Dimension of reference field in RA, Dec direction
C      DX, DY   R     Size of reference field pixels in RA, Dec directions
C      DIST     R     Radius of search beyond field edge (degrees).
C   Outputs:
C      IRECNO   I     Number of record in file (first record is number 1).
C                     <= 0 if no field is found.
C      NAME     C*12  The name of the field.
C      RA       D     RA of field (degrees, J2000).
C      DEC      D     Dec of field (degrees, J2000).
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER          UNIT, IRECNO, IERR, NX, NY
      REAL             DX, DY, DIST
      DOUBLE PRECISION RA0, DEC0, RA, DEC
      CHARACTER*12     NAME
C
      REAL DTOT, XSIZE, YSIZE
      INTEGER ILO, IHI, IMID
      DOUBLE PRECISION DECLIM
      INCLUDE 'FIELDS.INC'
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
C Search to find first field that might overlap reference field.
C Note that catalog is sorted in order of descending declination.
C
      XSIZE = DX*NX
      YSIZE = DY*NY
      DTOT = DIST+YSIZE/2
      DECLIM = DEC0+DTOT
C
C Use binary search
C
      ILO = 0
      IHI = FLDMAX
  10  CONTINUE
         IMID = (IHI+ILO)/2
C
C Read this field info
C
         CALL FLDGET (UNIT, IMID, NAME, RA, DEC, IERR)
         IF (IERR .NE. 0) RETURN
C
         IF (DEC.GT.DECLIM) THEN
            ILO = IMID
         ELSE
            IHI = IMID
         ENDIF
         IF (IHI.GT.ILO+1) GO TO 10
C
C OK, we know DECLIM is at IHI: make sure it is not outside
C range of catalog
C
      CALL FLDGET (UNIT, IHI, NAME, RA, DEC, IERR)
      IF (IERR .NE. 0) RETURN
C
      IF (DEC.LT.DEC0-DTOT .OR. DEC.GT.DEC0+DTOT) THEN
C
C No fields in range DEC0 +/- DTOT
C
         IRECNO = -1
         WRITE(MSGTXT,1000) DEC0
1000     FORMAT('Warning: declination',F10.5,
     *      ' is outside dec range of field catalog')
         CALL MSGWRT(1)
      ELSE
C
C Initialize list of positions along edges of reference field
C
         CALL EDGINT (RA0, DEC0, NX, NY, DX, DY, IERR)
         IF (IERR.NE.0) RETURN
C
C Find next field within distance DIST starting at IHI-1
C
         IRECNO = IHI-1
         CALL FLDNXT (UNIT, RA0, DEC0, NX, NY, DX, DY, DIST, IRECNO,
     *      NAME, RA, DEC, IERR)
         IF (IERR.NE.0) RETURN
      ENDIF
C
      IERR = 0
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE FLDNXT (UNIT, RA0, DEC0, NX, NY, DX, DY, DIST, IRECNO,
     *   NAME, RA, DEC, IERR)
C-----------------------------------------------------------------------
C   Find next field in catalog that is within DIST degrees of the edge
C   of field defined by center (RA0,DEC0), dimension (NX,NY),
C   and pixel scale (DX,DY).
C
C   Inputs:
C      UNIT     I     Unit number of (already opened) catalog.
C      RA0      D     Reference RA (J2000, degrees).
C      DEC0     D     Reference Declination (J2000, degrees).
C      NX, NY   R     Dimension of reference field in RA, Dec direction
C      DX, DY   R     Size of reference field pixels in RA, Dec direction
C      DIST     R     Radius of search beyond field edge (degrees).
C      IRECNO   I     Number of record in file where last field was
C                     found (first record is number 1).
C   Outputs:
C      IRECNO   I     Changed to indicate number of next record.
C                     <= 0 if no field is found.
C      NAME     C*12  The name of the field.
C      RA       D     RA of field (degrees, J2000).
C      DEC      D     Dec of field (degrees, J2000).
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
C
      INTEGER          UNIT, IRECNO, IERR, NX, NY
      REAL             DX, DY, DIST
      DOUBLE PRECISION RA0, DEC0, RA, DEC
      CHARACTER*12     NAME
C
      REAL XSIZE, YSIZE, GCIRCL, SEP
      REAL DECMIN, DTOT1, DTOT2
      INTEGER I, J
      INCLUDE 'FIELDS.INC'
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
C Just check each record consecutively starting with IRECNO+1
C
      IF (IRECNO .LT. 0) THEN
         MSGTXT = 'Error in CATNXT: record number <= 0'
         CALL MSGWRT (8)
         IERR = 1
         RETURN
      ENDIF
C
      XSIZE = DX*NX
      YSIZE = DY*NY
      DECMIN = DEC0 - (DIST+YSIZE/2)
      DTOT1  = DIST+MIN(XSIZE,YSIZE)/2
      DTOT2  = DIST+SQRT(XSIZE**2+YSIZE**2)/2
C
      DO 10 I=IRECNO+1, FLDMAX
         CALL FLDGET (UNIT, I, NAME, RA, DEC, IERR)
         IF (IERR.NE.0) RETURN
C See if we've gone past last possible match
         IF (DEC .LT. DECMIN) GO TO 20
C Check great circle distance
         SEP = GCIRCL(RA,DEC,RA0,DEC0)
         IF (SEP.LE.DTOT1) THEN
C Found one
             IRECNO = I
             RETURN
         ELSE IF (SEP.LE.DTOT2) THEN
C This is in the fuzzy distance range and might overlap corner, so
C check it more closely by computing great circle distance from edge
C points to RA,DEC.
             DO 30 J=1,NEDGE
                SEP = GCIRCL(RA,DEC,RAEDGE(J),DCEDGE(J))
                IF (SEP.LE.DIST) THEN
C OK, looks like this is close enough
                   IRECNO = I
                   RETURN
                ENDIF
  30         CONTINUE
         ENDIF
  10  CONTINUE
C
C No more matching records, return -1 for IRECNO
C
  20  CONTINUE
      IRECNO = -1
      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE EDGINT (RA0, DEC0, NX, NY, DX, DY, IERR)
C-----------------------------------------------------------------------
C   Compute RA and DEC at a set of positions along the edge of
C   this field and save in common.  These will be used to check
C   overlap of fields located in ambiguous areas.
C
C   Inputs:
C      RA0      D     Reference RA (J2000, degrees).
C      DEC0     D     Reference Declination (J2000, degrees).
C      NX, NY   R     Dimension of reference field in RA, Dec direction
C      DX, DY   R     Size of reference field pixels in RA, Dec direction
C   Outputs:
C      IERR     I     Error return code, 0=OK
C
C   RAEDGE, DCEDGE are set in common.
C-----------------------------------------------------------------------
C
      INTEGER          IERR, NX, NY
      REAL             DX, DY
      DOUBLE PRECISION RA0, DEC0
C
      INTEGER J, K
      DOUBLE PRECISION Z
      INCLUDE 'FIELDS.INC'
      INCLUDE 'INCS:DMSG.INC'
C
C-----------------------------------------------------------------------
C
      K = 0
      DO 10 J=1,NEDGE/4
         CALL XYVAL( 1.0+(J-1)*NX/10, 1.0,
     *       RAEDGE(K+1), DCEDGE(K+1), Z, IERR)
         IF (IERR.NE.0) RETURN
         CALL XYVAL( 1.0+J*NX/10, FLOAT(NY),
     *       RAEDGE(K+2), DCEDGE(K+2), Z, IERR)
         IF (IERR.NE.0) RETURN
         CALL XYVAL( 1.0, 1.0+J*NY/10,
     *       RAEDGE(K+3), DCEDGE(K+3), Z, IERR)
         IF (IERR.NE.0) RETURN
         CALL XYVAL( FLOAT(NX), 1.0+(J-1)*NY/10,
     *       RAEDGE(K+4), DCEDGE(K+4), Z, IERR)
         IF (IERR.NE.0) RETURN
         K = K+4
  10  CONTINUE
      RETURN
      END
