      SUBROUTINE GETPOS(INPUT, RACEN, DECEN, XSHIFT, YSHIFT, IERR)
C-----------------------------------------------------------------------
C   Get RA and Dec for phase center of map.  This is a fudged version
C   for the Green Bank maps, which do not have the OBSRA and OBSDEC
C   keywords set.  Unfortunately OGET(..., 'POSITION.OBSRA'...) goes
C   ahead and returns a value even when none is set.  This version
C   searches through the FITS header keywords to find RA.. and DEC...
C
C   Inputs:
C      INPUT   C*   Name of input image object.
C   Outputs:
C      RACEN   D    RA phase center (degrees)
C      DECEN   D    Dec phase center (degrees)
C      XSHIFT  R    X shift from phase center (degrees)
C      YSHIFT  R    Y shift from phase center (degrees)
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      CHARACTER INPUT*(*)
      DOUBLE PRECISION    RACEN, DECEN
      INTEGER   IERR
C
      INTEGER TYPE, DIM(7), I
      REAL XSHIFT, YSHIFT
      DOUBLE PRECISION CRVAL(7)
      CHARACTER*8 CTYPE(7)
      CHARACTER*56 CLONG
      EQUIVALENCE (CTYPE, CLONG)
C
C
      INCLUDE 'INCS:PAOOF.INC'
      INCLUDE 'INCS:DMSG.INC'
C-----------------------------------------------------------------------
      IERR = 0
C Get type for each axis
      CALL OGET (INPUT, 'IMAGE_DESC.CTYPE', TYPE, DIM,
     *   CLONG, CLONG, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL OGET (INPUT, 'IMAGE_DESC.CRVAL', TYPE, DIM,
     *   CRVAL, CRVAL, IERR)
      IF (IERR.NE.0) GO TO 999
C
C Look for 'RA' or 'RA---SIN' and 'DEC' or 'DEC--SIN'
C
      RACEN = -999.0
      DECEN = -999.0
      DO 10 I=1,7
         IF (CTYPE(I).EQ.'RA      ' .OR. CTYPE(I).EQ.'RA---SIN') THEN
C Found RA
            RACEN = CRVAL(I)
         ENDIF
         IF (CTYPE(I).EQ.'DEC     ' .OR. CTYPE(I).EQ.'DEC--SIN') THEN
C Found DEC
            DECEN = CRVAL(I)
         ENDIF
  10  CONTINUE
      IF (RACEN.EQ.-999.0 .OR. DECEN.EQ.-999.0) THEN
         MSGTXT = 'Could not find RA and DEC in CRTYPE keywords'
         CALL MSGWRT (6)
         IERR = 1
         GO TO 999
      ENDIF
C
 999  RETURN
      END
