      SUBROUTINE GETEPC(INPUT, JEPOCH, IERR)
C-----------------------------------------------------------------------
C   Get epoch for map
C   Inputs:
C      INPUT   C*   Name of input image object.
C   Outputs:
C      JEPOCH  R    Equinox (e.g. 1993.35)
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      CHARACTER INPUT*(*)
      REAL      JEPOCH
      INTEGER   IERR
C
      CHARACTER CDATE*8
      INTEGER TYPE, DIM(7), MM, DD, YY, OFFSET(12), DOFY
C
      INCLUDE 'INCS:DMSG.INC'
C
C OFFSET is cumulative number of days to first of each month
      DATA OFFSET/0, 31, 59, 90, 120, 151, 182, 212, 243, 273,
     *   304, 334/
C-----------------------------------------------------------------------
      JEPOCH = 0.0
      CALL OGET (INPUT,'IMAGE_DESC.DATE-OBS',TYPE,DIM,CDATE,CDATE,IERR)
      IF (IERR.NE.0) GO TO 999
C
C Convert string to epoch
C
      IF (CDATE(3:3) .EQ. '/' .AND. CDATE(6:6) .EQ. '/') THEN
C Old (pre-Y2K) format DD/MM/YY
         READ (CDATE, 1001) DD, MM, YY
         IF (YY .LT. 50) THEN
C Year is apparently greater than 2000
            YY = YY+2000
         ELSE
C Year is apparently less than 2000
            YY = YY+1900
         ENDIF
         WRITE (MSGTXT, 1003) 'Old DATE-OBS format', CDATE, YY, MM, DD
         CALL MSGWRT (1)
      ELSE IF (CDATE(5:5) .EQ. '-' .AND. CDATE(8:8) .EQ. '-') THEN
C Flaky format YYYY-MM- (with no day of month)
         READ (CDATE, 1004) YY, MM
         DD = 15
         WRITE (MSGTXT, 1003) 'Flaky DATE-OBS format', CDATE, YY, MM, DD
         CALL MSGWRT (1)
      ELSE IF (CDATE(3:3) .NE. '/' .AND. CDATE(6:6) .NE. '/') THEN
C New (post-Y2K) format YYYYMMDD
         READ (CDATE, 1002) YY, MM, DD
         WRITE (MSGTXT, 1003) 'New DATE-OBS format', CDATE, YY, MM, DD
         CALL MSGWRT (1)
      ELSE
         WRITE(MSGTXT,1000) CDATE
         CALL MSGWRT (4)
         GO TO 999
      ENDIF
C Check for sensible values
      IF (YY .LT. 1980 .OR. YY .GT. 2020 .OR. MM .LT. 1 .OR. MM .GT. 12
     *     .OR. DD .LT. 1 .OR. DD .GT. 31) THEN
         WRITE(MSGTXT,1000) CDATE
         CALL MSGWRT (4)
         GO TO 999
      ENDIF
C
C
      DOFY = OFFSET(MM) + DD
      IF (MOD(YY, 4) .EQ. 0) THEN
C a leap year
         IF (MM .GE. 3) DOFY = DOFY+1
         JEPOCH = YY + (DOFY-0.5)/366.0
      ELSE
         JEPOCH = YY + (DOFY-0.5)/365.0
      ENDIF
C
 999  CONTINUE
      RETURN
C
1000  FORMAT('Format of DATE-OBS is illegal: "',A,'"')
1001  FORMAT(I2,1X,I2,1X,I2)
1002  FORMAT(I4,I2,I2)
1003  FORMAT(A,1X,A,1X,I4,'/',I2,'/',I2)
1004  FORMAT(I4,1X,I2)
      END
