C From bcotton@gorilla.cv.nrao.edu Tue May 11 11:06:29 1993
C To: Rick White <rlw@stsci.edu>
C Subject: Re: trouble updating history in OOP
C 
C I've fixed a number of problems that could be related to the
C problems you're having.  I'm appending the current version of
C $APLOOP/HISTORY.FOR which you can try.  There is always a danger in
C moving software from one context to another but there's a reasonable
C chance this will cure your problems.  If there are problems with
C linking with this let me know what the missing routines are and I can
C send them as well.
C -Bill
C
C
C   History Utility module
C-----------------------------------------------------------------------
C! Object Oriented AIPS Fortran history utility routines
C# History Utility Object-Oriented
C   This software is the subject of a User agreement and is confidential
C   in nature. It shall not be sold or otherwise made available or
C   disclosed to third parties.
C-----------------------------------------------------------------------
C   This module contains routines which can be used to manipulate
C   histories associated with objects that are disk resident.
C   Available functions are:
C
C   OHCOPY (in, out, iret)
C      Copies the history from object in the object out.  iret=0
C      indicates success.
C   OHWRIT (entry, out, iret)
C      Writes up to 72 characters from character string entry to the
C      history associated with object out.  iret=0 indicates success.
C   OHLIST (in, list, nlist, out, iret)
C      Writes the names and values of the members of object in specified
C      by the first nlist elements in the character string array list to
C      the history associated with object out.  iret=0 indicates
C      success.
C   OHTIME (out, iret)
C      Adds task name and time and date stamp to history associated with
C      object out.  iret=0 indicates success.  Assures that a history
C      file exists. 
C-----------------------------------------------------------------------
      SUBROUTINE OHCOPY (IN, OUT, IRET)
C-----------------------------------------------------------------------
C   Copies the history from object in the object out.
C   If the underlying files are the same then no copy is made.
C   Inputs:
C      IN     C*?  Input object name.
C      OUT    C*?  Output object name.
C   Output:
C      IRET   I    Return code, 0=OK else an error occurred.
C-----------------------------------------------------------------------
      CHARACTER IN*(*), OUT*(*)
      INTEGER   IRET
C
      INTEGER   DISKI, DISKO, CNOI, CNOO, LUNI, LUNO, BUFNOI, BUFNOO,
     *   CAT(256), IERR, MSGSAV
      LOGICAL   NOCOPY
      INCLUDE 'INCS:OBJPARM.INC'
      INCLUDE 'INCS:CLASSIO.INC'
      INCLUDE 'INCS:DMSG.INC'
C-----------------------------------------------------------------------
      IRET = 0
C                                       Open objects and get info.
      CALL OBOPEN (IN, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Which buffer
      CALL OBINFO (IN, BUFNOI, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       DISK, CNO
      CALL OBDSKC (IN, DISKI, CNOI, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       LUN
      CALL OBLUN (LUNI, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Output
      CALL OBOPEN (OUT, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Which buffer
      CALL OBINFO (OUT, BUFNOO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       DISK, CNO
      CALL OBDSKC (OUT, DISKO, CNOO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       LUN
      CALL OBLUN (LUNO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Sync header
C??    CALL OBCSYN (OUT, IERR)
C??    IF (IRET.NE.0) GO TO 990
C                                       Catalog header
      CALL OBHGET (OUT, CAT, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Initialize history routines
      CALL HIINIT (3)
C                                       Don't do copy it the two have
C                                       the same underlying files.
      NOCOPY = (DISKI.EQ.DISKO) .AND. (CNOI.EQ.CNOO)
C                                       Don't copy if output file exists
      MSGSAV = MSGSUP
      MSGSUP = 32000
      CALL HIOPEN (LUNO, DISKO, CNOO, OBUFFR(1,BUFNOO), IERR)
      NOCOPY = NOCOPY .OR. (IERR.EQ.0)
      IF (IERR.EQ.0) CALL HICLOS (LUNO, .FALSE., OBUFFR(1,BUFNOO), IERR)
      MSGSUP = MSGSAV
C                                       Don't copy if input file doesn't
C                                       exist 
      MSGSAV = MSGSUP
      MSGSUP = 32000
      CALL HIOPEN (LUNI, DISKI, CNOI, OBUFFR(1,BUFNOI), IERR)
      NOCOPY = NOCOPY .OR. (IERR.NE.0)
      IF (IERR.EQ.0) CALL HICLOS (LUNI, .FALSE., OBUFFR(1,BUFNOI), IERR)
      MSGSUP = MSGSAV
C                                       Copy
      IF (.NOT.NOCOPY) THEN
C                                       Copy history
         CALL HISCOP (LUNI, LUNO, DISKI, DISKO, CNOI, CNOO, CAT,
     *      OBUFFR(1,BUFNOI), OBUFFR(1,BUFNOO), IRET)
         IF (IRET.NE.0) GO TO 990
C                                       Close
         CALL HICLOS (LUNO, .TRUE., OBUFFR(1,BUFNOO), IRET)
         IF (IERR.NE.0) IRET = IERR
C                                       Update Catalog header
         CALL OBHPUT (OUT, CAT, IRET)
         IF (IRET.NE.0) GO TO 990
         END IF
C                                       Free LUNs
      CALL OBLUFR (LUNI)
      CALL OBLUFR (LUNO)
C                                       Close objects
      CALL OBCLOS (IN, IRET)
      IF (IRET.NE.0) GO TO 990
      CALL OBCLOS (OUT, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Put time label if no copy was
C                                       made.
      IF (NOCOPY) THEN
         CALL OHTIME (OUT, IRET)
         IF (IRET.NE.0) GO TO 990
         END IF
      GO TO 999
C                                       Error
 990  MSGTXT = 'ERROR COPYING HISTORY'
      CALL  MSGWRT (6)
C
 999  RETURN
      END
      SUBROUTINE OHWRIT (ENTRY, OUT, IRET)
C-----------------------------------------------------------------------
C   Writes up to 72 characters from character string entry to the
C   history associated with object out.
C   Inputs:
C      ENTRY  C*?   Text of history entry; comments should begin with a
C                   "/" .  Up to 72 characters used.
C      OUT    C*?   Output object name.
C   Output:
C      IRET   I     Return code, 0=OK else an error occurred.
C-----------------------------------------------------------------------
      CHARACTER ENTRY*(*), OUT*(*)
      INTEGER   IRET
C
      INTEGER   DISKO, CNOO, LUNO, BUFNOO, L, IERR
      CHARACTER HILINE*72
      INCLUDE 'INCS:OBJPARM.INC'
      INCLUDE 'INCS:CLASSIO.INC'
      INCLUDE 'INCS:DMSG.INC'
      INCLUDE 'INCS:DHIS.INC'
C-----------------------------------------------------------------------
      IRET = 0
C                                       Output
      CALL OBOPEN (OUT, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Which buffer
      CALL OBINFO (OUT, BUFNOO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       DISK, CNO
      CALL OBDSKC (OUT, DISKO, CNOO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       LUN
      CALL OBLUN (LUNO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Initialize history routines
      CALL HIINIT (3)
C                                       Copy history
      CALL HIOPEN (LUNO, DISKO, CNOO, OBUFFR(1,BUFNOO), IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Add entry
      L = MIN (72, LEN (ENTRY))
      HILINE = TSKNAM //'  '// ENTRY(1:L)
      CALL HIADD (LUNO, HILINE, OBUFFR(1,BUFNOO), IERR)
C                                       Close
      CALL HICLOS (LUNO, .TRUE., OBUFFR(1,BUFNOO), IRET)
      IF (IERR.NE.0) IRET = IERR
C                                       Free LUN
      CALL OBLUFR (LUNO)
C                                       Close objects
      CALL OBCLOS (OUT, IRET)
      IF (IRET.NE.0) GO TO 990
      GO TO 999
C                                       Error
 990  MSGTXT = 'ERROR WRITING HISTORY FOR ' // OUT
      CALL  MSGWRT (6)
C      
 999  RETURN
      END
      SUBROUTINE OHLIST (IN, LIST, NLIST, OUT, IRET)
C-----------------------------------------------------------------------
C   Writes the names and values of the members of object in specified
C   by the first nlist elements in the character string array list to
C   the history associated with object out.
C   Inputs:
C      IN     C*?     Input object name.
C      LIST   C(?)*?  List of members of input object IN to copy to
C                     history of object OUT.
C      NLIST  I       Number of elements in LIST
C      OUT    C*?     Output object name.
C   Output:
C      IRET   I       Return code, 0=OK else an error occurred.
C-----------------------------------------------------------------------
      INTEGER   NLIST, IRET
      CHARACTER IN*(*), LIST(NLIST)*(*), OUT*(*)
C
      INTEGER   DISKO, CNOO, LUNO, BUFNOO, TYPE, DIM(7), I, ND1, ND2,
     *   ID1, ID2, IVAL(50), L, LC, INDEX, IC1, IC2, IERR, ITRIM
      REAL      RVAL(50)
      LOGICAL   BLNKED
      DOUBLE PRECISION DVAL(50)
      CHARACTER HILINE*72, LINE*120, CVAL*1000, CS*72
      EQUIVALENCE (IVAL, RVAL, DVAL)
      INCLUDE 'INCS:OBJPARM.INC'
      INCLUDE 'INCS:CLASSIO.INC'
      INCLUDE 'INCS:PAOOF.INC'
      INCLUDE 'INCS:DMSG.INC'
      INCLUDE 'INCS:DHIS.INC'
C-----------------------------------------------------------------------
      IRET = 0
C                                       Output
      CALL OBOPEN (OUT, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Which buffer
      CALL OBINFO (OUT, BUFNOO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       DISK, CNO
      CALL OBDSKC (OUT, DISKO, CNOO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       LUN
      CALL OBLUN (LUNO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Initialize history routines
      CALL HIINIT (3)
C                                       Copy history
      CALL HIOPEN (LUNO, DISKO, CNOO, OBUFFR(1,BUFNOO), IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Add entries
      DO 100 I = 1,NLIST
C                                       Fetch entry
         CALL OGET (IN, LIST(I), TYPE, DIM, IVAL, CVAL, IERR)
         IF (IERR.NE.0) GO TO 200
C                                       Member name length
         L = ITRIM (LIST(I))
C                                       Get dimensionality
         ND1 = MAX (1, DIM(1))
         ND2 = MAX (1, DIM(2))
C                                       Characters are slightly
C                                       different. 
         IF (TYPE.EQ.OOACAR) THEN
            ND1 = MAX (1, DIM(2))
            ND2 = MAX (1, DIM(3))
            END IF
         DO 50 ID2 = 1,ND2
            DO 40 ID1 = 1,ND1
               LINE = ' '
               INDEX = ID1 + (ID2-1) * ND1
C                                       Process by data type
               IF (TYPE.EQ.OOADP) THEN
C                                       Double precision
C                                       Don't write zeros.
                  BLNKED = ABS (DVAL(INDEX)) .LT. 1.0E-30
C                                       Get proper dimensionality
                  IF (ND2.GT.1) THEN
                     WRITE (LINE,1000) LIST(I)(1:L), ID1, ID2,
     *                  DVAL(INDEX) 
                  ELSE IF (ND1.GT.1) THEN
                     WRITE (LINE,1001) LIST(I)(1:L), ID1, DVAL(INDEX)
                  ELSE
                     WRITE (LINE,1002) LIST(I)(1:L), DVAL(INDEX)
                     END IF
               ELSE IF (TYPE.EQ.OOARE) THEN
C                                       Real
C                                       Don't write zeros.
                  BLNKED = ABS (RVAL(INDEX)) .LT. 1.0E-30
C                                       Get proper dimensionality
                  IF (ND2.GT.1) THEN
                     WRITE (LINE,1003) LIST(I)(1:L), ID1, ID2,
     *                  RVAL(INDEX) 
                  ELSE IF (ND1.GT.1) THEN
                     WRITE (LINE,1004) LIST(I)(1:L), ID1, RVAL(INDEX)
                  ELSE
                     WRITE (LINE,1005) LIST(I)(1:L), RVAL(INDEX)
                     END IF
               ELSE IF (TYPE.EQ.OOACAR) THEN
C                                       Character
                  IC1 = 1 + (INDEX-1) * DIM(1)
                  IC2 = IC1 + DIM(1) - 1
                  CS = CVAL(IC1:IC2)
C                                       Don't write blanks
                  BLNKED = CS .EQ. '      '
                  LC = ITRIM (CS)
C                                       Get proper dimensionality
                  IF (ND2.GT.1) THEN
                     WRITE (LINE,1006) LIST(I)(1:L), ID1, ID2, CS(1:LC)
                  ELSE IF (ND1.GT.1) THEN
                     WRITE (LINE,1007) LIST(I)(1:L), ID1, CS(1:LC)
                  ELSE
                     WRITE (LINE,1008) LIST(I)(1:L), CS(1:LC)
                     END IF
               ELSE IF (TYPE.EQ.OOAINT) THEN
C                                       Integer
C                                       Don't write zeros.
                  BLNKED = IVAL(INDEX) .EQ. 0
C                                       Get proper dimensionality
                  IF (ND2.GT.1) THEN
                     WRITE (LINE,1009) LIST(I)(1:L), ID1, ID2,
     *                  IVAL(INDEX) 
                  ELSE IF (ND1.GT.1) THEN
                     WRITE (LINE,1010) LIST(I)(1:L), ID1, IVAL(INDEX)
                  ELSE
                     WRITE (LINE,1011) LIST(I)(1:L), IVAL(INDEX)
                     END IF
               ELSE
C                                       Unknown skip
                  BLNKED = .TRUE.
                  END IF
               IF (.NOT.BLNKED) THEN
                  HILINE = TSKNAM //'  '// LINE(1:72)
                  CALL HIADD (LUNO, HILINE, OBUFFR(1,BUFNOO), IERR)
                  IF (IERR.NE.0) GO TO 200
                  END IF
 40            CONTINUE
 50         CONTINUE
 100     CONTINUE
C                                       Close
 200  CALL HICLOS (LUNO, .TRUE., OBUFFR(1,BUFNOO), IRET)
      IF (IERR.NE.0) IRET = IERR
C                                       Free LUN
      CALL OBLUFR (LUNO)
C                                       Close objects
      CALL OBCLOS (OUT, IRET)
      IF (IRET.NE.0) GO TO 990
      GO TO 999
C                                       Error
 990  MSGTXT = 'ERROR WRITING HISTORY FOR ' // OUT
      CALL  MSGWRT (6)
C      
 999  RETURN
C-----------------------------------------------------------------------
 1000 FORMAT (A,'(',I3,',',I3,') =', 1PD20.10)
 1001 FORMAT (A,'(',I3,') =', 1PD20.10)
 1002 FORMAT (A,' =', 1PD20.10)
 1003 FORMAT (A,'(',I3,',',I3,') =', 1PE12.5)
 1004 FORMAT (A,'(',I3,') =', 1PE12.5)
 1005 FORMAT (A,' =', 1PE12.5)
 1006 FORMAT (A,'(',I3,',',I3,') =''',A,'''')
 1007 FORMAT (A,'(',I3,') =''',A,'''')
 1008 FORMAT (A,' =''',A,'''')
 1009 FORMAT (A,'(',I3,',',I3,') =', I8)
 1010 FORMAT (A,'(',I3,') =', I8)
 1011 FORMAT (A,' =', I8)
      END
      SUBROUTINE OHTIME (OUT, IRET)
C-----------------------------------------------------------------------
C   Adds task name and time and date stamp to history associated with
C   object out.  Assures that a history file exists.
C   Inputs:
C      OUT    C*?   Output object name.
C   Output:
C      IRET   I     Return code, 0=OK else an error occurred.
C-----------------------------------------------------------------------
      CHARACTER OUT*(*)
      INTEGER   IRET
C
      INTEGER   DISKO, CNOO, LUNO, BUFNOO, IERR, CAT(256), MSGSAV,
     *   TIME(3), DATE(3)
      CHARACTER HILINE*72, CTIME*8, CDATE*12
      INCLUDE 'INCS:OBJPARM.INC'
      INCLUDE 'INCS:CLASSIO.INC'
      INCLUDE 'INCS:DMSG.INC'
      INCLUDE 'INCS:DDCH.INC'
      INCLUDE 'INCS:DHIS.INC'
C-----------------------------------------------------------------------
      IRET = 0
C                                       Output
      CALL OBOPEN (OUT, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Which buffer
      CALL OBINFO (OUT, BUFNOO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       DISK, CNO
      CALL OBDSKC (OUT, DISKO, CNOO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       LUN
      CALL OBLUN (LUNO, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Sync header
C??    CALL OBCSYN (OUT, IERR)
C??    IF (IRET.NE.0) GO TO 990
C                                       Catalog header
      CALL OBHGET (OUT, CAT, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Initialize history routines
      CALL HIINIT (3)
C                                       Open/create history - suppress
C                                       warning messages.
      MSGSAV = MSGSUP
      MSGSUP = 32000
      CALL HICREA (LUNO, DISKO, CNOO, CAT, OBUFFR(1,BUFNOO), IRET)
      MSGSUP = MSGSAV
      IF (IRET.NE.0) GO TO 990
C                                       Write time and date
      CALL ZDATE (DATE)
      CALL ZTIME (TIME)
      CALL TIMDAT (TIME, DATE, CTIME, CDATE)
      WRITE (HILINE,1100) TSKNAM, RLSNAM, CDATE, CTIME
      CALL HIADD (LUNO, HILINE, OBUFFR(1,BUFNOO), IERR)
C                                       Close
      CALL HICLOS (LUNO, .TRUE., OBUFFR(1,BUFNOO), IRET)
      IF (IERR.NE.0) IRET = IERR
C                                       Save catalog header in case HI
C                                       file created.
      CALL OBHPUT (OUT, CAT, IRET)
      IF (IRET.NE.0) GO TO 990
C                                       Free LUN
      CALL OBLUFR (LUNO)
C                                       Close objects
      CALL OBCLOS (OUT, IRET)
      IF (IRET.NE.0) GO TO 990
      GO TO 999
C                                       Error
 990  MSGTXT = 'ERROR WRITING HISTORY FOR ' // OUT
      CALL  MSGWRT (6)
C      
 999  RETURN
C-----------------------------------------------------------------------
 1100 FORMAT (A6,'RELEASE =''',A7,' ''  /********* Start ',A12,2X,A8)
      END
