      REAL FUNCTION PBEAM (R, FREQ, IERR)
C-----------------------------------------------------------------------
C   Returns primary beam response as function of position.
C
C   Inputs:
C      R        R     Distance from phase center (degrees)
C      FREQ     R     Frequency of observation (Hz)
C   Outputs:
C      Returns  R     Primary beam response at X,Y normalized so
C                     that PB at X=Y=0 is 1.
C      IERR     I     Error return code, 0=OK
C-----------------------------------------------------------------------
      REAL      R, FREQ
      INTEGER   IERR
C
      REAL RS, BESSJ1
C
C-----------------------------------------------------------------------
C
C                                       Dimensionless distance off-axis
      RS = 6.4653596 * R * ( FREQ / 1.385E9 )
      IF (RS .LE. 1.E-8) THEN
         PBEAM = 1.0
      ELSE
         PBEAM = (2.0*BESSJ1(RS)/RS)**2
      ENDIF
      IERR = 0
      RETURN
      END

      REAL FUNCTION BESSJ1(X)
C Bessel function J1 from Numerical Recipes
      REAL X
C
      REAL AX, Z, XX
      DOUBLE PRECISION
     *   Y,P1,P2,P3,P4,P5,Q1,Q2,Q3,Q4,Q5,R1,R2,R3,R4,R5,R6,
     *   S1,S2,S3,S4,S5,S6
      DATA R1,R2,R3,R4,R5,R6/72362614232.D0, -7895059235.D0,
     *     242396853.1D0, -2972611.439D0, 15704.48260D0,
     *    -30.16036606D0/,
     *   S1,S2,S3,S4,S5,S6/144725228442.D0, 2300535178.D0,
     *      18583304.74D0, 99447.43394D0, 376.9991397D0,
     *      1.D0/
      DATA P1,P2,P3,P4,P5/1.D0, .183105D-2, -.3516396496D-4,
     *      .2457520174D-5, -.240337019D-6/,
     *   Q1,Q2,Q3,Q4,Q5/.04687499995D0, -.2002690873D-3,
     *      .8449199096D-5, -.88228987D-6, .105787412D-6/
C
      IF(ABS(X).LT.8.)THEN
         Y=X**2
         BESSJ1=X*(R1+Y*(R2+Y*(R3+Y*(R4+Y*(R5+Y*R6)))))
     *      /(S1+Y*(S2+Y*(S3+Y*(S4+Y*(S5+Y*S6)))))
      ELSE
         AX=ABS(X)
         Z=8./AX
         Y=Z**2
         XX=AX-2.356194491
         BESSJ1=SQRT(.636619772/AX)*(COS(XX)*(P1+Y*(P2+Y*(P3+Y*(P4+Y
     *      *P5))))-Z*SIN(XX)*(Q1+Y*(Q2+Y*(Q3+Y*(Q4+Y*Q5)))))
     *      *SIGN(1.,X)
      ENDIF
      RETURN
      END
