      SUBROUTINE PRECSS(RA, DEC, INEQX, OUTEQX, IERR)
C
C Conversion between B1950 and J2000 coordinates
C RA,DEC are double precision coordinates (degrees) in
C input equinox INEQX.  On output they are the corresponding
C coordinates in OUTEQX.  INEQX and OUTEQX must be selected
C from 2000 and 1950.
C
      DOUBLE PRECISION RA, DEC
      INTEGER INEQX, OUTEQX, IERR
C
      INCLUDE 'INCS:DMSG.INC'
C
      IF (INEQX.EQ.1950 .AND. OUTEQX.EQ.2000) THEN
         CALL TO2000(RA, DEC)
      ELSE IF (INEQX.EQ.2000 .AND. OUTEQX.EQ.1950) THEN
         CALL TO1950(RA, DEC)
      ELSE
         MSGTXT =
     *   'PRECSS: Input and output equinoxes must be 1950 or 2000'
         CALL MSGWRT (6)
         IERR = 1
      ENDIF
      RETURN
      END

      SUBROUTINE TO2000(RA, DEC)
C-----------------------------------------------------------------------
C   Convert RA and Dec from B1950 to J2000 coordinates.
C
C   Inputs:
C      RA      D    RA in B1950 (degrees)
C      DEC     D    Dec in B1950 (degrees)
C   Outputs:
C      RA      D    RA in J2000 (degrees)
C      DEC     D    Dec in J2000 (degrees)
C-----------------------------------------------------------------------
      DOUBLE PRECISION    RA, DEC
C
      INTEGER I, J
      DOUBLE PRECISION A(3), ADOT(3), M(6,6), R0(3), R(6), A1(3),
     *   AR, CD, T1, T, PI, D2R
C
      DATA PI/3.14159265358979323846264/
C
      DATA A/-1.62557D-6, -0.31919D-6, -0.13843D-6/,
     *  ADOT/ 1.244D-3,   -1.579D-3,   -0.660D-3 /
C
      DATA M/0.9999256782, -0.0111820611,     -0.0048579477, 
     *   0.00000242395018, -0.00000002710663, -0.00000001177656,
     *   0.0111820610,      0.9999374784,     -0.0000271765,
     *   0.00000002710663,  0.00000242397878, -0.00000000006587,
     *   0.0048579479,     -0.0000271474,      0.9999881997,
     *   0.00000001177656, -0.00000000006582,  0.00000242410173,
     *  -0.000551,         -0.238565,          0.435739,
     *   0.99994704,       -0.01118251,       -0.00485767,
     *   0.238514,         -0.002667,         -0.008541,
     *   0.01118251,        0.99995883,       -0.00002718,
     *  -0.435623,          0.012254,          0.002117,
     *   0.00485767,       -0.00002714,        1.00000956/
C
C-----------------------------------------------------------------------
C
C Precess from B1950 to J2000
C
C From Aoki, X., Soma, M., Kinoshita, H., Inoue, K. (1983) A.Ap. 128, 263.
C and Astronomical Almanac, 1989, pp. B42-B43.
C
C Simplified for the case of no proper motions or parallaxes
C in J2000 coordinate system.  Note that a fixed target would
C have a spurious proper motion (the E-term of the aberration)
C in the B1950 coordinates, so for fixed targets the correct assumption
C is no PM in the J2000 frame.
C
      D2R = PI/180.D0
      CD = COS(DEC*D2R)
      R0(1) = CD*COS(RA*D2R)
      R0(2) = CD*SIN(RA*D2R)
      R0(3) = SIN(DEC*D2R)
C
C correct for E-terms of aberration
C assume epoch of observation was about 1980; there ought to be
C another version of the program when the epoch of the observations
C are known.
C
C T1 = (time from 1950 measured in centuries) * arcsecs
C T  = (time from 2000 measured in centuries) * arcsecs
C
      T1 = (1980.0D0 - 1950.0D0)/100.0D0
      T = ( T1 - 0.5D0 )*(D2R/3600.0D0)
      T1 = T1 * (D2R/3600.0D0)
      AR = 0.0
      DO 10 I=1,3
         A1(I) = A(I) + ADOT(I)*T1
         AR = AR + R0(I)*A1(I)
  10  CONTINUE
      DO 30 I=1,3
         R0(I) = (1.0D0+AR)*R0(I) - A1(I)
  30  CONTINUE

C precess from 1950 to 2000
      DO 40 I=1,6
          R(I) = 0.0D0
          DO 40 J=1,3
             R(I) = R(I) + M(J,I)*R0(J)
  40  CONTINUE

C modify r for assumed 1980 epoch
      DO 50 I=1,3
         R(I) = R(I) + R(I+3)*T
  50  CONTINUE

C convert from x,y,z to RA, Dec
      DEC = ASIN( R(3) / SQRT(R(1)*R(1)+R(2)*R(2)+R(3)*R(3)) )
      RA  = ATAN2(R(2),R(1))
C convert back to degrees
      DEC = DEC / D2R
      RA = RA / D2R
C
      RETURN
      END

      SUBROUTINE TO1950(RA, DEC)
C-----------------------------------------------------------------------
C   Convert RA and Dec from J2000 to B1950 coordinates.
C
C   Inputs:
C      RA      D    RA in J2000 (degrees)
C      DEC     D    Dec in J2000 (degrees)
C   Outputs:
C      RA      D    RA in B1950 (degrees)
C      DEC     D    Dec in B1950 (degrees)
C-----------------------------------------------------------------------
      DOUBLE PRECISION    RA, DEC
C
      INTEGER I, J, K
      DOUBLE PRECISION A(3), ADOT(3), MINV(6,6), M(6,6),
     *   RP(3), R0(3), R1(3), R(6), S(3), A1(3),
     *   AS, RTOT, CD, T1, T, PI, D2R
C
      DATA PI/3.14159265358979323846264/
C
      DATA A/-1.62557D-6, -0.31919D-6, -0.13843D-6/,
     *  ADOT/ 1.244D-3,   -1.579D-3,   -0.660D-3 /
C
      DATA M/0.9999256782, -0.0111820611,     -0.0048579477, 
     *   0.00000242395018, -0.00000002710663, -0.00000001177656,
     *   0.0111820610,      0.9999374784,     -0.0000271765,
     *   0.00000002710663,  0.00000242397878, -0.00000000006587,
     *   0.0048579479,     -0.0000271474,      0.9999881997,
     *   0.00000001177656, -0.00000000006582,  0.00000242410173,
     *  -0.000551,         -0.238565,          0.435739,
     *   0.99994704,       -0.01118251,       -0.00485767,
     *   0.238514,         -0.002667,         -0.008541,
     *   0.01118251,        0.99995883,       -0.00002718,
     *  -0.435623,          0.012254,          0.002117,
     *   0.00485767,       -0.00002714,        1.00000956/
C
      DATA MINV/0.9999256795,  0.0111814828,      0.0048590039,
     *  -0.00000242389840,    -0.00000002710544, -0.00000001177742, 
     *  -0.0111814828,         0.9999374849,     -0.0000271771,
     *   0.00000002710544,    -0.00000242392702,  0.00000000006585,
     *  -0.0048590040,        -0.0000271557,      0.9999881946,
     *   0.00000001177742,     0.00000000006585, -0.00000242404995,
     *  -0.000551,             0.238509,         -0.435614,
     *   0.99990432,           0.01118145,        0.00485852,
     *  -0.238560,            -0.002667,          0.012254,
     *  -0.01118145,           0.99991613,       -0.00002717,
     *   0.435730,            -0.008541,          0.002117,
     *  -0.00485852,          -0.00002716,        0.99996684/
C
C
C-----------------------------------------------------------------------
C
C Precess from J2000 to B1950
C
C From Aoki, X., Soma, M., Kinoshita, H., Inoue, K. (1983) A.Ap. 128, 263.
C and Astronomical Almanac, 1989, pp. B42-B43.
C
      D2R = PI/180.D0
      CD = COS(DEC*D2R)
      RP(1) = CD*COS(RA*D2R)
      RP(2) = CD*SIN(RA*D2R)
      RP(3) = SIN(DEC*D2R)
C
C precess from 2000 to 1950
C assume epoch of observation was about 1980; there ought to be
C another version of the program when the epoch of the observations
C are known.
C
C T1 = (time from 1950 measured in centuries) * arcsecs
C T  = (time from 2000 measured in centuries) * arcsecs
C
      T1 = (1980.0D0 - 1950.0D0)/100.0D0
      T = ( T1 - 0.5D0 )*(D2R/3600.0D0)
      T1 = T1 * (D2R/3600.0D0)
C
C first guess is r=rp, v=0
C
      DO 10 I=1,3
        R(I) = RP(I)
        R(I+3) = 0.0
  10  CONTINUE
C
C have to iterate to get proper motion in r1 frame
C
      DO 70 K=1,3
         DO 30 I=1,3
            R1(I) = 0.0
            DO 20 J=1,6
               R1(I) = R1(I) + MINV(J,I)*R(J)
  20        CONTINUE
  30     CONTINUE
C
C calculate new velocity
C
         DO 50 I=4,6
            R(I) = 0.0
            DO 40 J=1,3
               R(I) = R(I) + M(J,I)*R1(J)
  40        CONTINUE
  50     CONTINUE
C
C calculate new position with this v
C
         DO 60 I=1,3
            R(I) = RP(I) - R(I+3)*T
  60     CONTINUE
  70  CONTINUE
C
C final version of r1
C
      DO 90 I=1,3
         R1(I) = 0.0
         DO 80 J=1,6
            R1(I) = R1(I) + MINV(J,I)*R(J)
  80     CONTINUE
  90  CONTINUE

C
C correct for E-terms of aberration
C
      DO 100 I=1,3
         A1(I) = A(I) + ADOT(I)*T1
 100  CONTINUE
      RTOT = SQRT(R1(1)*R1(1) + R1(2)*R1(2) + R1(3)*R1(3))
      DO 110 I=1,3
         S(I) = R1(I)/RTOT
 110  CONTINUE
C
C iterate twice to converge
C
      DO 150 J=1,3
         AS = 0.0
         DO 120 I=1,3
            AS = AS + S(I)*A1(I)
 120     CONTINUE
         DO 130 I=1,3
            R0(I) = R1(I) + A1(I) - AS*S(I)
 130     CONTINUE
C new value of s
         RTOT = SQRT(R0(1)*R0(1) + R0(2)*R0(2) + R0(3)*R0(3))
         DO 140 I=1,3
            S(I) = R0(I)/RTOT
 140     CONTINUE
 150  CONTINUE

C convert from x,y,z to RA, Dec
      DEC = ASIN( S(3) / SQRT(S(1)*S(1)+S(2)*S(2)+S(3)*S(3)) )
      RA  = ATAN2(S(2),S(1))
C convert back to degrees
      DEC = DEC / D2R
      RA = RA / D2R
C
      RETURN
      END
