      SUBROUTINE READCC (INTAB, MAXCC, NCC, XPOS, YPOS, FLUX,
     *   WORK, IWORK, IERR)
C-----------------------------------------------------------------------
C   Read clean compoments.  Components are read in blocks and are
C   merged.  The number of merged components must be less than MAXCC.
C
C   Inputs:
C      INTAB   C*   Name of input table object. (Already opened)
C      MAXCC   I    Maximum number of merged CCs.
C   Output:
C      NCC     I    Number of merged CCs.
C      XPOS    R    X component positions (degrees)
C      YPOS    R    Y component positions (degrees)
C      FLUX    R    Component fluxes (Jy)
C      WORK    R    Work space
C      IWORK   I    Work space
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      CHARACTER INTAB*(*)
      INTEGER   IERR, MAXCC, NCC
      REAL XPOS(MAXCC), YPOS(MAXCC), FLUX(MAXCC), WORK(MAXCC),
     *   IWORK(MAXCC)
C
C
      INTEGER IROW, NROW, TYPE, DIM(7)
      INTEGER I1, I2, IREAD
C
      INCLUDE 'INCS:PAOOF.INC'
      INCLUDE 'INCS:DMSG.INC'
C-----------------------------------------------------------------------
C                                       Get number of entries
      CALL OGET (INTAB, 'NROW', TYPE, DIM, NROW, NROW, IERR)
      IF (IERR.NE.0) GO TO 999
C                                       Read table to internal arrays.
C                                       We read a little at a time an
C                                       merge it so we can read large
C                                       tables if necessary.
      NCC = 0
      IREAD = 0
      I1 = 1
      I2 = MIN(NROW,MAXCC)
 100  IF (I1.GT.I2) GO TO 300
         DO 200 IROW = I1,I2
            NCC = NCC + 1
            IREAD = IREAD + 1
            CALL CCTGET (INTAB, IROW, XPOS(NCC), YPOS(NCC), FLUX(NCC),
     *         IERR)
            IF (IERR.GT.0) GO TO 999
C                                       Flagged?
            IF (IERR.LT.0) NCC = NCC - 1
 200     CONTINUE
C                                       Merge clean components
         CALL CCMRGE (XPOS, YPOS, FLUX, NCC, WORK, IWORK, IERR)
         IF (IERR.NE.0) GO TO 999
         I1 = I2+1
         I2 = MIN(NROW,MAXCC-NCC)
      GO TO 100
 300  CONTINUE
C                                       Check to make we got all CCs
      IF (NROW .NE. IREAD) THEN
         WRITE (MSGTXT,1000) NROW, NCC
         CALL MSGWRT (8)
         IERR = 5
         GO TO 999
      ENDIF
C
      IF (NCC.NE.NROW) THEN
         WRITE (MSGTXT,1001) NROW, NCC
         CALL MSGWRT (1)
      ENDIF
C
 999  RETURN
C-----------------------------------------------------------------------
 1000 FORMAT ('TOO MANY COMPONENTS FOR INTERNAL ARRAYS:', I7, '>', I8)
 1001 FORMAT('Merging reduced CC from',I7,' to',I7)
      END

      SUBROUTINE CCTGET (NAME, ROW, X, Y, FLUX, IERR)
C-----------------------------------------------------------------------
C   Get row from CC (CLEAN component) table object.
C   This assumes the structure of the CC table
C   Inputs:
C      NAME    C*?  CC table object name.
C      ROW     I    Row number
C   Output:
C      X       R    X coordinate
C      Y       R    Y coordinate
C      FLUX    R    Component flux density.
C      IERR    I    Return error code, 0=>OK
C-----------------------------------------------------------------------
      CHARACTER NAME*(*)
      INTEGER   ROW, IERR
      REAL      X, Y, FLUX
C
      INTEGER   DIM(7), TYPE
C-----------------------------------------------------------------------
C                                       Read
      CALL TABDGT (NAME, ROW, 1, TYPE, DIM, FLUX, FLUX, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL TABDGT (NAME, ROW, 2, TYPE, DIM, X, X, IERR)
      IF (IERR.NE.0) GO TO 999
      CALL TABDGT (NAME, ROW, 3, TYPE, DIM, Y, Y, IERR)
C
 999  RETURN
      END

      SUBROUTINE CCMRGE (XPOS, YPOS, FLUX, NCC, WORK, IWORK, IERR)
C-----------------------------------------------------------------------
C   Merge clean components at same positions in list.
C   Inputs:
C      XPOS    R    CC X position (degrees)
C      YPOS    R    CC Y position (degrees)
C      FLUX    R    CC flux
C      NCC     I    Number of clean components
C   Outputs:
C      XPOS    R    X,Y,F are modified if components are merged.
C      YPOS    R      Returned arrays are sorted by increasing X, Y, F.
C      FLUX    R
C      NCC     I    Number of clean components is modified too
C      WORK    R    Work array of length NCC
C      IWORK   I    Work array of length NCC
C-----------------------------------------------------------------------
      INTEGER NCC, IERR
      REAL XPOS(NCC), YPOS(NCC), FLUX(NCC), WORK(NCC)
      INTEGER IWORK(NCC)
C
C
C
      INTEGER I, J, NEWNCC
      INCLUDE 'INCS:DMSG.INC'
C
C                                       Sort by decreasing X, decreasing Y
      CALL SORT3(NCC,XPOS,YPOS,FLUX,WORK,IWORK,IERR)
      IF (IERR.NE.0) GO TO 999
C
C                                       Merge components
C
C Loops are WHILE (I.LE.NCC) and WHILE (J.LE.NCC)
C
      I = 1
      NEWNCC = 0
100   IF (I.GT.NCC) GO TO 400
         NEWNCC = NEWNCC+1
         XPOS(NEWNCC) = XPOS(I)
         YPOS(NEWNCC) = YPOS(I)
         FLUX(NEWNCC) = FLUX(I)
         J = I+1
200      IF (J.GT.NCC) GO TO 300
            IF (XPOS(J).EQ.XPOS(I) .AND. YPOS(J).EQ.YPOS(I)) THEN
C                                       Identical positions, merge them
               FLUX(NEWNCC) = FLUX(NEWNCC)+FLUX(J)
            ELSE
C                                       Not identical, go on to next point
               GO TO 300
            ENDIF
            J = J+1
            GO TO 200
300      CONTINUE
         I = J
         GO TO 100
400   CONTINUE
C
C                                       Now reset NCC
      NCC = NEWNCC
      IF (IERR.NE.0) GO TO 999
C
999   RETURN
      END
