      SUBROUTINE SKY2XY(RA,DEC,RACEN,DECEN,X,Y,IERR)
C-----------------------------------------------------------------------
C   Convert RA,DEC in degrees to X,Y in degrees from phase center using
C   standard SIN projection.
C
C   Inputs:
C      RA      D    RA in degrees
C      DEC     D    Declination in degrees
C      RACEN   D    Phase center RA in degrees
C      DECEN   D    Phase center Declination in degrees
C   Outputs:
C      X       R    X position in degrees
C      Y       R    Y position in degrees
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      REAL      X, Y
      DOUBLE PRECISION    RACEN, DECEN, RA, DEC
      INTEGER   IERR
C
      DOUBLE PRECISION PI, D2R, CDEC
      PARAMETER (PI=3.14159265358979323846264)
C
C-----------------------------------------------------------------------
C
C This does not include the "droop" (3-d sky curvature effect on
C source positions) but does include the curvature that results from
C projecting the sphere onto the tangent plane.  Once we have access
C to the zenith angle and parallactic angle for each observation should
C put in droop too.
C
      D2R = PI/180.0D0
      CDEC = COS(D2R*DEC)
      X = (
     *   CDEC * SIN(D2R*(RA-RACEN))
     *   ) / D2R
      Y = (
     *   -SIN(D2R*DECEN) * CDEC * COS(D2R*(RA-RACEN)) +
     *   COS(D2R*DECEN) * SIN(D2R*DEC)
     *   ) / D2R
      IERR = 0
999   RETURN
      END
