      SUBROUTINE XY2SKY(X,Y,RACEN,DECEN,XSHIFT,YSHIFT,RA,DEC,IERR)
C-----------------------------------------------------------------------
C   Convert X,Y in degrees from phase center to RA, DEC in degrees using
C   standard SIN projection.
C
C   Inputs:
C      X       R    X position in degrees
C      Y       R    Y position in degrees
C      RACEN   D    Phase center RA in degrees
C      DECEN   D    Phase center Declination in degrees
C      XSHIFT  R    X shift of field from phase center (degrees)
C      YSHIFT  R    Y shift of field from phase center (degrees)
C   Outputs:
C      RA      D    RA in degrees
C      DEC     D    Declination in degrees
C      IERR    I    Error code: 0 => ok
C-----------------------------------------------------------------------
      REAL      X, Y, XSHIFT, YSHIFT
      DOUBLE PRECISION    RACEN, DECEN, RA, DEC
      INTEGER   IERR
C
      DOUBLE PRECISION PI, D2R, XT, YT, ZT, C0, S0
      PARAMETER (PI=3.14159265358979323846264)
C
C-----------------------------------------------------------------------
C
C This does not include the "droop" (3-d sky curvature effect on
C source positions) but does include the curvature that results from
C projecting the sphere onto the tangent plane.  Once we have access
C to the zenith angle and parallactic angle for each observation should
C put in droop too.
C
C The XSHIFT and YSHIFT parameters will be needed to put in the 3-d
C distortion effects, which are a function of the distance away from
C the field center (rather than from the phase center specified
C by RACEN and DECEN.)
C
      D2R = PI/180.0D0
      XT = D2R*X
      YT = D2R*Y
      ZT = SQRT(1.D0-XT*XT-YT*YT)
      C0 = COS(D2R*DECEN)
      S0 = SIN(D2R*DECEN)
      DEC = ASIN(YT*C0+ZT*S0)/D2R
      RA = RACEN + ATAN2( XT, ZT*C0-YT*S0 )/D2R
      IERR = 0
      RETURN
      END
